@@ -1,5 +1,5 @@
 /* as.c - GAS main program.
-   Copyright (C) 1987, 1990, 1991, 1992 Free Software Foundation, Inc.
+   Copyright (C) 1987, 1990, 1991, 1992, 1994 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -32,6 +32,9 @@
  *
  */
 
+#include "ansidecl.h"
+#include "libiberty.h"
+
 #include <stdio.h>
 #include <string.h>
 
@@ -43,7 +46,6 @@
 #define COMMON
 
 #include "as.h"
-#include "config.h"
 #include "subsegs.h"
 #include "output-file.h"
 
@@ -73,6 +75,11 @@ char *myname;			/* argv[0] */
 segT reg_section, expr_section;
 segT text_section, data_section, bss_section;
 #endif
+
+/* This is true if the assembler should output time and space usage. */
+
+static int statistics_flag = 0;
+
 
 void
 print_version_id ()
@@ -98,6 +105,8 @@ main (argc, argv)
   char **work_argv;		/* variable copy of argv */
   char *arg;			/* an arg to program */
   char a;			/* an arg flag (after -) */
+  int keep_it;
+  long start_time = get_run_time ();
 
 #if 0 /* do we need any of this?? */
   {
@@ -144,13 +153,32 @@ main (argc, argv)
 
       if (*arg != '-')		/* Filename. We need it later. */
 	continue;		/* Keep scanning args looking for flags. */
-      if (arg[1] == '-' && arg[2] == 0)
+      /* Handle double-dash options. */
+      if (arg[1] == '-')
 	{
-	  /* "--" as an argument means read STDIN */
-	  /* on this scan, we don't want to think about filenames */
-	  *work_argv = "";	/* Code that means 'use stdin'. */
+	  if (arg[2] == 0)
+	    {
+	      /* "--" as an argument means read stdin. */
+	      /* On this scan, we don't want to think about filenames. */
+	      *work_argv = "";	/* A code that means 'use stdin'. */
+	    }
+	  else if (strcmp (arg, "--statistics") == 0)
+	    {
+	      statistics_flag = 1;
+	      *work_argv = NULL;
+	    }
+#ifdef md_parse_long_option
+	  else if (md_parse_long_option (arg))
+	    *work_argv = NULL;
+#endif
+	  else
+	    {
+	      as_warn ("Unknown option `%s' ignored", arg);
+	      *work_argv = NULL;
+	    }
 	  continue;
 	}
+
       /* This better be a switch. */
       arg++;			/*->letter. */
 
@@ -266,6 +294,16 @@ main (argc, argv)
 	      arg = "";		/* Finished with this arg. */
 	      break;
 
+	    case 'n':
+	      if (*arg && strcmp(arg, "ocpp") == 0)
+		;
+	      else
+		{
+		  as_warn ("Unknown option `-n%s' ignored", arg);
+		  arg += strlen (arg);
+		  break;
+		}
+
 	    case 'R':
 	      /* -R means put data into text segment */
 	      flag_readonly_data_in_text = 1;
@@ -330,32 +368,57 @@ main (argc, argv)
   assert (stdoutput != 0);
 #endif
 
+#ifdef tc_init_after_args
+  tc_init_after_args ();
+#endif
+
   /* Here with flags set up in flagseen[]. */
   perform_an_assembly_pass (argc, argv);	/* Assemble it. */
 #ifdef TC_I960
   brtab_emit ();
 #endif
+
   if (seen_at_least_1_file ()
       && !((had_warnings () && flag_always_generate_output)
 	   || had_errors () > 0))
-    {
-      write_object_file ();	/* relax() addresses then emit object file */
-    }				/* we also check in write_object_file() just before emit. */
-#ifdef BFD_ASSEMBLER
+    keep_it = 1;
   else
-    {
-      output_file_close (out_file_name);
-      unlink (out_file_name);
-    }
-#endif
+    keep_it = 0;
 
-  input_scrub_end ();
-  md_end ();			/* MACHINE.c */
+  if (keep_it)
+    write_object_file ();
 
 #ifndef NO_LISTING
   listing_print ("");
 #endif
 
+#ifndef OBJ_VMS /* does its own file handling */
+#ifndef BFD_ASSEMBLER
+  if (keep_it)
+#endif
+    output_file_close (out_file_name);
+#endif
+
+  if (!keep_it)
+    unlink (out_file_name);
+
+  input_scrub_end ();
+#ifdef md_end
+  md_end ();
+#endif
+
+  if (statistics_flag)
+    {
+      extern char **environ;
+      char *lim = (char *) sbrk (0);
+      long run_time = get_run_time () - start_time;
+
+      fprintf (stderr, "%s: total time in assembly: %ld.%06ld\n",
+	       myname, run_time / 1000000, run_time % 1000000);
+      fprintf (stderr, "%s: data size %ld\n",
+	       myname, (long) (lim - (char *) &environ));
+    }
+
   if ((had_warnings () && flagseen['Z'])
       || had_errors () > 0)
     return EXIT_FAILURE;
@@ -397,12 +460,19 @@ perform_an_assembly_pass (argc, argv)
   {
     segT seg;
 
+#ifdef TE_APOLLO
+    seg = subseg_new (".wtext", 0);
+#else
     seg = subseg_new (".text", 0);
+#endif
     assert (seg == SEG_E0);
     seg = subseg_new (".data", 0);
     assert (seg == SEG_E1);
     seg = subseg_new (".bss", 0);
     assert (seg == SEG_E2);
+#ifdef TE_APOLLO
+    create_target_segments ();
+#endif
   }
 
 #else /* not MANY_SEGMENTS */
@@ -426,6 +496,7 @@ perform_an_assembly_pass (argc, argv)
   bfd_set_section_flags (stdoutput, data_section,
 			 applicable & (SEC_ALLOC | SEC_LOAD | SEC_RELOC));
   bfd_set_section_flags (stdoutput, bss_section, applicable & SEC_ALLOC);
+  seg_info (bss_section)->bss = 1;
   subseg_new (BFD_ABS_SECTION_NAME, 0);
   subseg_new (BFD_UND_SECTION_NAME, 0);
   reg_section = subseg_new ("*GAS `reg' section*", 0);
