@@ -31,7 +31,11 @@
 #define BYTES_IN_WORD 4		/* for now */
 #include "../bfd/libelf.h"
 
-#define TARGET_SYMBOL_FIELDS	int local:1; unsigned long sy_name_offset;
+/* Symbol fields used by the ELF back end.  */
+#define ELF_TARGET_SYMBOL_FIELDS int local:1; unsigned long sy_name_offset;
+
+/* Don't change this; change ELF_TARGET_SYMBOL_FIELDS instead.  */
+#define TARGET_SYMBOL_FIELDS ELF_TARGET_SYMBOL_FIELDS
 
 #include "targ-cpu.h"
 
@@ -49,31 +53,25 @@
 
 extern asection *gdb_section;
 
-#if 0 /* This should not reference i!!
-	 If it's really needed, fix it, and the call site if necessary.  */
-#define obj_frob_symbol(S,PUNT)	if ( obj_elf_frob_symbol (S, &PUNT) ) { i++; continue; }
-#endif
-
 /* Copy over the function bit and size of a forwarded symbol.  */
 #define obj_frob_forward_symbol(sym)					\
   (((sym)->bsym->flags |=						\
     ((sym)->sy_value.X_add_symbol->bsym->flags & BSF_FUNCTION)),	\
    S_SET_SIZE ((sym), S_GET_SIZE ((sym)->sy_value.X_add_symbol)))
 
-#define obj_write_symbol(S)	obj_elf_write_symbol (S)
-
 #define obj_frob_file()	elf_frob_file()
 
-extern int obj_elf_frob_symbol PARAMS ((struct symbol *, int *));
 extern void elf_frob_file PARAMS ((void));
 extern void elf_file_symbol PARAMS ((char *));
 
-extern int obj_elf_write_symbol PARAMS ((struct symbol *));
-
 extern void obj_elf_section PARAMS ((int));
 extern void obj_elf_previous PARAMS ((int));
 extern void obj_elf_version PARAMS ((int));
 
+/* BFD wants to write the udata field, which is a no-no for the
+   globally defined sections.  */
+#define obj_sec_sym_ok_for_reloc(SEC)	((SEC)->owner != 0)
+
 /* Stabs go in a separate section.  */
 #define SEPARATE_STAB_SECTIONS
 
@@ -82,4 +80,32 @@ extern void obj_elf_version PARAMS ((int));
 extern void obj_elf_init_stab_section PARAMS ((segT));
 #define INIT_STAB_SECTION(seg) obj_elf_init_stab_section (seg)
 
+/* For now, always set ECOFF_DEBUGGING for a MIPS target.  */
+#ifdef TC_MIPS
+#define ECOFF_DEBUGGING
+#endif
+
+#ifdef ECOFF_DEBUGGING
+
+/* If we are generating ECOFF debugging information, we need some
+   additional fields for each symbol.  */
+#undef TARGET_SYMBOL_FIELDS
+#define TARGET_SYMBOL_FIELDS \
+  ELF_TARGET_SYMBOL_FIELDS \
+  struct efdr *ecoff_file; \
+  struct localsym *ecoff_symbol; \
+  valueT ecoff_extern_size;
+
+/* We smuggle stabs in ECOFF rather than using a separate section.
+   The Irix linker can not handle a separate stabs section.  */
+#undef SEPARATE_STAB_SECTIONS
+#undef INIT_STAB_SECTION
+#define OBJ_PROCESS_STAB(what, string, type, other, desc) \
+  ecoff_stab ((what), (string), (type), (other), (desc))
+
+/* ECOFF requires that we call the ecoff_frob_symbol hook.  */
+#define obj_frob_symbol(symp, punt) ecoff_frob_symbol (symp)
+
+#endif /* ECOFF_DEBUGGING */
+
 #endif /* _OBJ_ELF_H */
