@@ -41,6 +41,7 @@ extern char const_flag;
 extern const short seg_N_TYPE[];
 extern const segT N_TYPE_seg[];
 
+#undef NO_RELOC
 enum reloc_type
   {
     NO_RELOC, RELOC_32
@@ -108,26 +109,7 @@ struct nlist
 
 #define N_STAB	0340		/* Mask for all bits used for SDB entries 	*/
 
-#define	N_GSYM	0x20		/* global symbol: name,,0,type,0 */
-#define	N_FNAME	0x22		/* procedure name (f77 kludge): name,,0 */
-#define	N_FUN	0x24		/* procedure: name,,0,linenumber,address */
-#define	N_STSYM	0x26		/* static symbol: name,,0,type,address */
-#define	N_LCSYM	0x28		/* .lcomm symbol: name,,0,type,address */
-#define	N_RSYM	0x40		/* register sym: name,,0,type,register */
-#define	N_SLINE	0x44		/* src line: 0,,0,linenumber,address */
-#define	N_CATCH	0x54		/* */
-#define	N_SSYM	0x60		/* structure elt: name,,0,type,struct_offset */
-#define	N_SO	0x64		/* source file name: name,,0,0,address */
-#define	N_LSYM	0x80		/* local sym: name,,0,type,offset */
-#define	N_SOL	0x84		/* #included file name: name,,0,0,address */
-#define	N_PSYM	0xa0		/* parameter: name,,0,type,offset */
-#define	N_ENTRY	0xa4		/* alternate entry: name,linenumber,address */
-#define	N_LBRAC	0xc0		/* left bracket: 0,,0,nesting level,address */
-#define	N_RBRAC	0xe0		/* right bracket: 0,,0,nesting level,address */
-#define	N_BCOMM	0xe2		/* begin common: name,, */
-#define	N_ECOMM	0xe4		/* end common: name,, */
-#define	N_ECOML	0xe8		/* end common (local name): ,,address */
-#define	N_LENG	0xfe		/* second stab entry with length information */
+#include "aout/stab_gnu.h"
 
 /* SYMBOL TABLE */
 /* Symbol table entry data type */
@@ -439,11 +421,13 @@ extern int vms_resolve_symbol_redef ();
 #define DBG_S_C_UCHAR		0x02
 #define DBG_S_C_USINT		0x03
 #define DBG_S_C_ULINT		0x04
+#define DBG_S_C_UQUAD		0x05
 #define DBG_S_C_SCHAR		0x06
 #define DBG_S_C_SSINT		0x07
 #define DBG_S_C_SLINT		0x08
+#define DBG_S_C_SQUAD		0x09
 #define DBG_S_C_REAL4		0x0a
-#define DBG_S_C_REAL8		0x0b
+#define DBG_S_C_REAL8		0x0b		/* D_float (G_float is 0x1b) */
 #define DBG_S_C_FUNCTION_ADDR	0x17
 #define DBG_S_C_ADVANCED_TYPE	0xa3
 /*  These are the codes that are used to generate the definitions of struct
