@@ -33,7 +33,7 @@
 
 const char comment_chars[] = "!";
 const char line_separator_chars[] = ";";
-const char line_comment_chars[] = "!";
+const char line_comment_chars[] = "!#";
 
 /* This table describes all the machine specific pseudo-ops the assembler
    has to support.  The fields are:
@@ -58,7 +58,7 @@ const pseudo_typeS md_pseudo_table[] =
   {0, 0, 0}
 };
 
-int md_reloc_size;
+const int md_reloc_size;
 
 const char EXP_CHARS[] = "eE";
 
@@ -87,7 +87,7 @@ const char FLT_CHARS[] = "rRsSfFdDxXpP";
 #define WORD_F 32767
 #define WORD_B 32768
 
-relax_typeS md_relax_table[C (END, 0)];
+const relax_typeS md_relax_table[C (END, 0)];
 
 static struct hash_control *opcode_hash_control;	/* Opcode mnemonics */
 
@@ -102,6 +102,7 @@ md_begin ()
   h8500_opcode_info *opcode;
   char prev_buffer[100];
   int idx = 0;
+  register relax_typeS *table;
 
   opcode_hash_control = hash_new ();
   prev_buffer[0] = 0;
@@ -116,36 +117,38 @@ md_begin ()
 	}
     }
 
-  /* Initialize the relax table */
-  md_relax_table[C (BRANCH, BYTE_DISP)].rlx_forward = BYTE_F;
-  md_relax_table[C (BRANCH, BYTE_DISP)].rlx_backward = BYTE_B;
-  md_relax_table[C (BRANCH, BYTE_DISP)].rlx_length = 2;
-  md_relax_table[C (BRANCH, BYTE_DISP)].rlx_more = C (BRANCH, WORD_DISP);
+  /* Initialize the relax table.  We use a local variable to avoid
+     warnings about modifying a supposedly const data structure.  */
+  table = (relax_typeS *) md_relax_table;
+  table[C (BRANCH, BYTE_DISP)].rlx_forward = BYTE_F;
+  table[C (BRANCH, BYTE_DISP)].rlx_backward = BYTE_B;
+  table[C (BRANCH, BYTE_DISP)].rlx_length = 2;
+  table[C (BRANCH, BYTE_DISP)].rlx_more = C (BRANCH, WORD_DISP);
 
-  md_relax_table[C (BRANCH, WORD_DISP)].rlx_forward = WORD_F;
-  md_relax_table[C (BRANCH, WORD_DISP)].rlx_backward = WORD_B;
-  md_relax_table[C (BRANCH, WORD_DISP)].rlx_length = 3;
-  md_relax_table[C (BRANCH, WORD_DISP)].rlx_more = 0;
+  table[C (BRANCH, WORD_DISP)].rlx_forward = WORD_F;
+  table[C (BRANCH, WORD_DISP)].rlx_backward = WORD_B;
+  table[C (BRANCH, WORD_DISP)].rlx_length = 3;
+  table[C (BRANCH, WORD_DISP)].rlx_more = 0;
 
-  md_relax_table[C (SCB_F, BYTE_DISP)].rlx_forward = BYTE_F;
-  md_relax_table[C (SCB_F, BYTE_DISP)].rlx_backward = BYTE_B;
-  md_relax_table[C (SCB_F, BYTE_DISP)].rlx_length = 3;
-  md_relax_table[C (SCB_F, BYTE_DISP)].rlx_more = C (SCB_F, WORD_DISP);
+  table[C (SCB_F, BYTE_DISP)].rlx_forward = BYTE_F;
+  table[C (SCB_F, BYTE_DISP)].rlx_backward = BYTE_B;
+  table[C (SCB_F, BYTE_DISP)].rlx_length = 3;
+  table[C (SCB_F, BYTE_DISP)].rlx_more = C (SCB_F, WORD_DISP);
 
-  md_relax_table[C (SCB_F, WORD_DISP)].rlx_forward = WORD_F;
-  md_relax_table[C (SCB_F, WORD_DISP)].rlx_backward = WORD_B;
-  md_relax_table[C (SCB_F, WORD_DISP)].rlx_length = 8;
-  md_relax_table[C (SCB_F, WORD_DISP)].rlx_more = 0;
+  table[C (SCB_F, WORD_DISP)].rlx_forward = WORD_F;
+  table[C (SCB_F, WORD_DISP)].rlx_backward = WORD_B;
+  table[C (SCB_F, WORD_DISP)].rlx_length = 8;
+  table[C (SCB_F, WORD_DISP)].rlx_more = 0;
 
-  md_relax_table[C (SCB_TST, BYTE_DISP)].rlx_forward = BYTE_F;
-  md_relax_table[C (SCB_TST, BYTE_DISP)].rlx_backward = BYTE_B;
-  md_relax_table[C (SCB_TST, BYTE_DISP)].rlx_length = 3;
-  md_relax_table[C (SCB_TST, BYTE_DISP)].rlx_more = C (SCB_TST, WORD_DISP);
+  table[C (SCB_TST, BYTE_DISP)].rlx_forward = BYTE_F;
+  table[C (SCB_TST, BYTE_DISP)].rlx_backward = BYTE_B;
+  table[C (SCB_TST, BYTE_DISP)].rlx_length = 3;
+  table[C (SCB_TST, BYTE_DISP)].rlx_more = C (SCB_TST, WORD_DISP);
 
-  md_relax_table[C (SCB_TST, WORD_DISP)].rlx_forward = WORD_F;
-  md_relax_table[C (SCB_TST, WORD_DISP)].rlx_backward = WORD_B;
-  md_relax_table[C (SCB_TST, WORD_DISP)].rlx_length = 10;
-  md_relax_table[C (SCB_TST, WORD_DISP)].rlx_more = 0;
+  table[C (SCB_TST, WORD_DISP)].rlx_forward = WORD_F;
+  table[C (SCB_TST, WORD_DISP)].rlx_backward = WORD_B;
+  table[C (SCB_TST, WORD_DISP)].rlx_length = 10;
+  table[C (SCB_TST, WORD_DISP)].rlx_more = 0;
 
 }
 
@@ -258,7 +261,6 @@ parse_exp (s, op, page)
 {
   char *save;
   char *new;
-  segT seg;
 
   save = input_line_pointer;
 
@@ -268,9 +270,13 @@ parse_exp (s, op, page)
       if (s[1] == 'p' && s[2] == 'a' && s[3] == 'g' && s[4] == 'e')
 	{
 	  s += 5;
-
 	  *page = 'p';
 	}
+      if (s[1] == 'h' && s[2] == 'i' && s[3] == '1' && s[4] == '6')
+	{
+	  s += 5;
+	  *page = 'h';
+	}
       else if (s[1] == 'o' && s[2] == 'f' && s[3] == 'f')
 	{
 	  s += 4;
@@ -280,26 +286,12 @@ parse_exp (s, op, page)
 
   input_line_pointer = s;
 
-  seg = expr (0, op);
+  expression (op);
+  if (op->X_op == O_absent)
+    as_bad ("missing operand");
   new = input_line_pointer;
   input_line_pointer = save;
-  if (SEG_NORMAL (seg))
-    return new;
-  switch (seg)
-    {
-    case SEG_ABSOLUTE:
-    case SEG_UNKNOWN:
-    case SEG_DIFFERENCE:
-    case SEG_BIG:
-    case SEG_REGISTER:
-      return new;
-    case SEG_ABSENT:
-      as_bad ("Missing operand");
-      return new;
-    default:
-      as_bad ("Don't understand operand of type %s", segment_name (seg));
-      return new;
-    }
+  return new;
 }
 
 typedef enum
@@ -353,12 +345,16 @@ skip_colonthing (sign, ptr, exp, def, size8, size16, size24)
 	{
 	  exp->type = IMM8;
 	}
+      else if (exp->page == 'h')
+	{
+	  exp->type = IMM16;
+	}
       else
 	{
 	  /* Let's work out the size from the context */
 	  int n = exp->exp.X_add_number;
 	  if (size8
-	      && exp->exp.X_seg == SEG_ABSOLUTE
+	      && exp->exp.X_op == O_constant
 	      && ((sign == exp_signed && (n >= -128 && n <= 127))
 		  || (sign == exp_unsigned && (n >= 0 && (n <= 255)))
 		  || (sign == exp_sandu && (n >= -128 && (n <= 255)))))
@@ -422,9 +418,10 @@ parse_reglist (src, op)
     }
   idx++;
   op->exp.X_add_symbol = 0;
-  op->exp.X_subtract_symbol = 0;
+  op->exp.X_op_symbol = 0;
   op->exp.X_add_number = mask;
-  op->exp.X_seg = SEG_ABSOLUTE;
+  op->exp.X_op = O_constant;
+  op->exp.X_unsigned = 1;
   op->type = IMM8;
   return idx;
 
@@ -787,7 +784,7 @@ get_specific (opcode, operands)
 	      break;
 	    case QIM:
 	      if (user->type == IMM8
-		  && user->exp.X_seg == SEG_ABSOLUTE
+		  && user->exp.X_op == O_constant
 		  &&
 		  (user->exp.X_add_number == -2
 		   || user->exp.X_add_number == -1
@@ -863,7 +860,7 @@ check (operand, low, high)
      int low;
      int high;
 {
-  if (operand->X_seg != SEG_ABSOLUTE
+  if (operand->X_op != O_constant
       || operand->X_add_number < low
       || operand->X_add_number > high)
     {
@@ -881,14 +878,12 @@ insert (output, index, exp, reloc, pcrel)
      int reloc;
      int pcrel;
 {
-  fix_new (frag_now,
-	   output - frag_now->fr_literal + index,
-	   4,			/* always say size is 4, but we know better */
-	   exp->X_add_symbol,
-	   exp->X_subtract_symbol,
-	   exp->X_add_number,
-	   pcrel,
-	   reloc);
+  fix_new_exp (frag_now,
+	       output - frag_now->fr_literal + index,
+	       4,	       	/* always say size is 4, but we know better */
+	       exp,
+	       pcrel,
+	       reloc);
 }
 
 void
@@ -982,9 +977,25 @@ build_bytes (opcode, operand)
 	    case FPIND_D8:
 	      insert (output, index, &displacement, R_H8500_IMM8, 0);
 	      break;
+
 	    case IMM16:
-	      insert (output, index, &immediate, immediate_inpage ?
-		      R_H8500_LOW16 : R_H8500_IMM16, 0);
+	      {
+		int p;
+		switch (immediate_inpage) {
+		case 'p':
+		  p = R_H8500_HIGH16;
+		  break;
+		case 'h':		
+		  p = R_H8500_HIGH16;
+		  break;
+		default:
+		  p = R_H8500_IMM16;
+		  break;
+		}
+		
+		insert (output, index, &immediate,p, 0);
+	      }
+		
 	      index++;
 	      break;
 	    case RLIST:
@@ -1113,7 +1124,7 @@ DEFUN (md_assemble, (str),
       return;
     }
 
-  input_line_pointer = get_operands (opcode, op_end, operand);
+  get_operands (opcode, op_end, operand);
   prev_opcode = opcode;
 
   opcode = get_specific (opcode, operand);
@@ -1154,11 +1165,6 @@ DEFUN (tc_headers_hook, (headers),
   printf ("call to tc_headers_hook \n");
 }
 
-void
-DEFUN_VOID (md_end)
-{
-}
-
 /* Various routines to kill one day */
 /* Equal to MAX_PRECISION in atof-ieee.c */
 #define MAX_LITTLENUMS 6
@@ -1219,7 +1225,7 @@ md_atof (type, litP, sizeP)
       md_number_to_chars (litP, (long) (*wordP++), sizeof (LITTLENUM_TYPE));
       litP += sizeof (LITTLENUM_TYPE);
     }
-  return "";			/* Someone should teach Dean about null pointers */
+  return 0;
 }
 
 int
@@ -1244,8 +1250,8 @@ tc_aout_fix_to_chars ()
 void
 md_create_short_jump (ptr, from_addr, to_addr, frag, to_symbol)
      char *ptr;
-     long from_addr;
-     long to_addr;
+     addressT from_addr;
+     addressT to_addr;
      fragS *frag;
      symbolS *to_symbol;
 {
@@ -1255,7 +1261,7 @@ md_create_short_jump (ptr, from_addr, to_addr, frag, to_symbol)
 void
 md_create_long_jump (ptr, from_addr, to_addr, frag, to_symbol)
      char *ptr;
-     long from_addr, to_addr;
+     addressT from_addr, to_addr;
      fragS *frag;
      symbolS *to_symbol;
 {
@@ -1367,7 +1373,7 @@ md_convert_frag (headers, fragP)
     case C (SCB_F, UNDEF_WORD_DISP):
     case C (SCB_TST, UNDEF_WORD_DISP):
       /* This tried to be relaxed, but didn't manage it, it now needs a
-       fix */
+	 fix */
       wordify_scb (buffer, &disp_size, &inst_size);
 
       /* Make a reloc */
@@ -1375,7 +1381,6 @@ md_convert_frag (headers, fragP)
 	       fragP->fr_fix + inst_size,
 	       4,
 	       fragP->fr_symbol,
-	       0,
 	       fragP->fr_offset,
 	       0,
 	       R_H8500_PCREL16);
@@ -1399,13 +1404,12 @@ md_convert_frag (headers, fragP)
       fragP->fr_fix += disp_size + inst_size;
       fragP->fr_var = 0;
     }
-
 }
 
-long
-DEFUN (md_section_align, (seg, size),
-       segT seg AND
-       long size)
+valueT
+md_section_align (seg, size)
+     segT seg ;
+     valueT size;
 {
   return ((size + (1 << section_alignment[(int) seg]) - 1) 
 	  & (-1 << section_alignment[(int) seg]));
@@ -1440,6 +1444,10 @@ md_apply_fix (fixP, val)
     case R_H8500_HIGH8:
       *buf++ = val >> 16;
       break;
+    case R_H8500_HIGH16:
+      *buf++ = val >> 24;
+      *buf++ = val >> 16;
+      break;
     case R_H8500_IMM24:
       *buf++ = (val >> 16);
       *buf++ = (val >> 8);
@@ -1506,28 +1514,15 @@ md_estimate_size_before_relax (fragP, segment_type)
 void
 md_number_to_chars (ptr, use, nbytes)
      char *ptr;
-     long use;
+     valueT use;
      int nbytes;
 {
-  switch (nbytes)
-    {
-    case 4:
-      *ptr++ = (use >> 24) & 0xff;
-    case 3:
-      *ptr++ = (use >> 16) & 0xff;
-    case 2:
-      *ptr++ = (use >> 8) & 0xff;
-    case 1:
-      *ptr++ = (use >> 0) & 0xff;
-      break;
-    default:
-      abort ();
-    }
+  number_to_chars_bigendian (ptr, use, nbytes);
 }
+
 long
 md_pcrel_from (fixP)
      fixS *fixP;
-
 {
   return fixP->fx_size + fixP->fx_where + fixP->fx_frag->fr_address;
 }
@@ -1638,4 +1633,13 @@ start_label (ptr)
   return 1;
 }
 
+
+int
+tc_coff_sizemachdep (frag)
+     fragS *frag;
+{
+  return md_relax_table[frag->fr_subtype].rlx_length;
+}
+
 /* end of tc-h8500.c */
+
