@@ -31,7 +31,7 @@
    already far too much object file format dependent code in this file.
    In theory this file should contain only exported functions, structures
    and data declarations common to all PA assemblers.  */
-   
+
 #ifndef _TC_HPPA_H
 #define _TC_HPPA_H
 
@@ -47,7 +47,7 @@
 #include "../bfd/elf32-hppa.h"
 #define TARGET_FORMAT "elf32-hppa"
 #endif
-  
+
 #ifdef OBJ_SOM
 #include "../bfd/som.h"
 #define TARGET_FORMAT "som"
@@ -64,17 +64,8 @@
 #define FAKE_LABEL_NAME "L$0\001"
 #define ASEC_NULL (asection *)0
 
-/* sym1 - sym2 is valid on the PA as long as sym2 is $global$.  */
-
-#define SEG_DIFF_ALLOWED
-  
-/* FIXME.  */
-#ifdef OBJ_ELF
-extern void elf_hppa_final_processing PARAMS ((void));
-extern void hppa_tc_symbol PARAMS ((bfd *, elf_symbol_type *, int));
-#endif
-
-extern void hppa_tc_make_sections PARAMS ((bfd *));
+/* Labels are not required to have a colon for a suffix.  */
+#define LABELS_WITHOUT_COLONS
 
 /* FIXME.  This should be static and declared in tc-hppa.c, but 
    pa_define_label gets used outside of tc-hppa.c via tc_frob_label.
@@ -84,6 +75,7 @@ extern void pa_define_label ();
 /* FIXME.  Types not available here, so they can't be PARAMized.  */
 extern void parse_cons_expression_hppa ();
 extern void cons_fix_new_hppa ();
+extern int hppa_force_relocation ();
 
 #define tc_frob_label(sym) pa_define_label (sym)
 
@@ -97,8 +89,6 @@ extern void cons_fix_new_hppa ();
 /* FIXME.  More things which are both HPPA and ELF specific.  There is 
    nowhere to put such things.  */
 #ifdef OBJ_ELF
-#define elf_tc_symbol		hppa_tc_symbol
-#define elf_tc_make_sections	hppa_tc_make_sections
 #define elf_tc_final_processing	elf_hppa_final_processing
 #endif
 
@@ -121,6 +111,11 @@ extern void cons_fix_new_hppa ();
 
 #define tc_fix_adjustable hppa_fix_adjustable
 
+/* Because of the strange PA calling conventions, it is sometimes
+   necessary to emit a relocation for a call even though it would
+   normally appear safe to handle it completely within GAS.  */
+#define TC_FORCE_RELOCATION(FIXP) hppa_force_relocation (FIXP)
+
 /* If a symbol is imported, but never used, then the symbol should
    *not* end up in the symbol table.  Likewise for absolute symbols
    with local scope.  */
@@ -129,5 +124,5 @@ extern void cons_fix_new_hppa ();
 	|| (S_GET_SEGMENT (sym) == &bfd_abs_section \
 	    && (sym->bsym->flags & BSF_EXPORT) == 0)) \
       punt = 1
-	
+
 #endif /* _TC_HPPA_H */
