@@ -1,5 +1,5 @@
 /* ns32k.c  -- Assemble on the National Semiconductor 32k series
-   Copyright (C) 1987, 1992 Free Software Foundation, Inc.
+   Copyright (C) 1987, 1992, 1993 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -362,20 +362,14 @@ char disp_test[] =
 char disp_size[] =
 {4, 1, 2, 0, 4};
 
-
-#if __STDC__ == 1
-
-static segT evaluate_expr (expressionS * resultP, char *ptr);
-static void md_number_to_disp (char *buf, long val, int n);
-static void md_number_to_imm (char *buf, long val, int n);
-
-#else /* not __STDC__ */
-
-static segT evaluate_expr ();
-static void md_number_to_disp ();
-static void md_number_to_imm ();
-
-#endif /* not __STDC__ */
+static void evaluate_expr PARAMS ((expressionS * resultP, char *ptr));
+static void md_number_to_disp PARAMS ((char *buf, long val, int n));
+static void md_number_to_imm PARAMS ((char *buf, long val, int n));
+static void fix_new_ns32k_exp PARAMS ((fragS *frag, int where, int size,
+				       expressionS *exp, int pcrel,
+				       char pcrel_adjust,
+				       char im_disp, bit_fixS *bit_fixP,
+				       char bsr));
 
 /* Parses a general operand into an addressingmode struct
 
@@ -656,14 +650,14 @@ get_addr_mode (ptr, addr_modeP)
   if (addr_modeP->mode == DEFAULT || addr_modeP->scaled_mode == -1)
     {
       /* resolve ambigious operands, this shouldn't
-		   be necessary if one uses standard NSC operand
-		   syntax. But the sequent compiler doesn't!!!
-		   This finds a proper addressinging mode if it
-		   is implicitly stated. See ns32k-opcode.h */
+	 be necessary if one uses standard NSC operand
+	 syntax. But the sequent compiler doesn't!!!
+	 This finds a proper addressinging mode if it
+	 is implicitly stated. See ns32k-opcode.h */
       (void) evaluate_expr (&exprP, ptr);	/* this call takes time Sigh! */
       if (addr_modeP->mode == DEFAULT)
 	{
-	  if (exprP.X_add_symbol || exprP.X_subtract_symbol)
+	  if (exprP.X_add_symbol || exprP.X_op_symbol)
 	    {
 	      addr_modeP->mode = desc->default_model;	/* we have a label */
 	    }
@@ -674,7 +668,7 @@ get_addr_mode (ptr, addr_modeP)
 	}
       else
 	{
-	  if (exprP.X_add_symbol || exprP.X_subtract_symbol)
+	  if (exprP.X_add_symbol || exprP.X_op_symbol)
 	    {
 	      addr_modeP->scaled_mode = desc->default_model;
 	    }
@@ -832,18 +826,17 @@ list_search (str, optionP, default_map)
   return 0;
 }
 
-static segT
+static void
 evaluate_expr (resultP, ptr)
      expressionS *resultP;
      char *ptr;
 {
   register char *tmp_line;
-  register segT segment;
+
   tmp_line = input_line_pointer;
   input_line_pointer = ptr;
-  segment = expression (&exprP);
+  expression (&exprP);
   input_line_pointer = tmp_line;
-  return (segment);
 }
 
 /* Convert operands to iif-format and adds bitfields to the opcode.
@@ -1182,7 +1175,6 @@ convert_iif ()
   char *inst_offset;
   char **inst_opcode;
   char *memP;
-  segT segment;
   int l;
   int k;
   int rem_size;			/* count the remaining bytes of instruction */
@@ -1223,7 +1215,6 @@ convert_iif ()
 				     (long) (memP - frag_now->fr_literal),
 				     size,
 				     0,
-				     0,
 				     iif.iifP[i].object,
 				     iif.iifP[i].pcrel,
 				     (char) size_so_far,	/*iif.iifP[i].pcrel_adjust,*/
@@ -1251,7 +1242,8 @@ convert_iif ()
 		case 2:	/* the object is a pointer at an expression, so unpack
 					   it, note that bignums may result from the expression
 					   */
-		  if ((segment = evaluate_expr (&exprP, (char *) iif.iifP[i].object)) == SEG_BIG || size == 8)
+		  evaluate_expr (&exprP, (char *) iif.iifP[i].object);
+		  if (exprP.X_op == O_big || size == 8)
 		    {
 		      if ((k = exprP.X_add_number) > 0)
 			{	/* we have a bignum ie a quad */
@@ -1292,23 +1284,21 @@ convert_iif ()
 		    }
 		  if (j ||
 		      exprP.X_add_symbol ||
-		      exprP.X_subtract_symbol ||
+		      exprP.X_op_symbol ||
 		      iif.iifP[i].pcrel)
 		    {		/* fixit */
 		      /* the expression was undefined due to an undefined label */
 		      /* create a fix so we can fix the object later */
 		      exprP.X_add_number += iif.iifP[i].object_adjust;
-		      fix_new_ns32k (frag_now,
-				     (long) (memP - frag_now->fr_literal),
-				     size,
-				     exprP.X_add_symbol,
-				     exprP.X_subtract_symbol,
-				     exprP.X_add_number,
-				     iif.iifP[i].pcrel,
-				     (char) size_so_far,	/*iif.iifP[i].pcrel_adjust,*/
-				     iif.iifP[i].im_disp,
-				     j,
-				     iif.iifP[i].bsr);	/* sequent hack */
+		      fix_new_ns32k_exp (frag_now,
+					 (long) (memP - frag_now->fr_literal),
+					 size,
+					 &exprP,
+					 iif.iifP[i].pcrel,
+					 (char) size_so_far,	/*iif.iifP[i].pcrel_adjust,*/
+					 iif.iifP[i].im_disp,
+					 j,
+					 iif.iifP[i].bsr);	/* sequent hack */
 
 		    }
 		  else
@@ -1340,29 +1330,27 @@ convert_iif ()
 					    */
 	      {
 		int temp;
-		segment = evaluate_expr (&exprP, (char *) iif.iifP[i].object);
-		if ((exprP.X_add_symbol || exprP.X_subtract_symbol) &&
+		evaluate_expr (&exprP, (char *) iif.iifP[i].object);
+		if ((exprP.X_add_symbol || exprP.X_op_symbol) &&
 		    !iif.iifP[i].pcrel)
 		  {		/* OVE: hack, clamp to 4 bytes */
 		    size = 4;	/* we dont wan't to frag this, use 4 so it reaches */
-		    fix_new_ns32k (frag_now,
-				   (long) (memP - frag_now->fr_literal),
-				   size,
-				   exprP.X_add_symbol,
-				   exprP.X_subtract_symbol,
-				   exprP.X_add_number,
-				   0,	/* never iif.iifP[i].pcrel, */
-				   (char) size_so_far,	/*iif.iifP[i].pcrel_adjust,*/
-				   1,	/* always iif.iifP[i].im_disp, */
-				   0, 0);
+		    fix_new_ns32k_exp (frag_now,
+				       (long) (memP - frag_now->fr_literal),
+				       size,
+				       &exprP,
+				       0,	/* never iif.iifP[i].pcrel, */
+				       (char) size_so_far,	/*iif.iifP[i].pcrel_adjust,*/
+				       1,	/* always iif.iifP[i].im_disp, */
+				       0, 0);
 		    memP += size;
 		    rem_size -= 4;
 		    break;	/* exit this absolute hack */
 		  }
 
-		if (exprP.X_add_symbol || exprP.X_subtract_symbol)
+		if (exprP.X_add_symbol || exprP.X_op_symbol)
 		  {		/* frag it */
-		    if (exprP.X_subtract_symbol)
+		    if (exprP.X_op_symbol)
 		      {		/* We cant relax this case */
 			as_fatal ("Can't relax difference");
 		      }
@@ -1465,7 +1453,7 @@ md_begin ()
   inst_hash_handle = hash_new ();
   for (ptr = ns32k_opcodes; ptr < endop; ptr++)
     {
-      if (*(stat = hash_insert (inst_hash_handle, ptr->name, (char *) ptr)))
+      if ((stat = hash_insert (inst_hash_handle, ptr->name, (char *) ptr)))
 	{
 	  as_fatal ("Can't hash %s: %s", ptr->name, stat);	/*fatal*/
 	  exit (0);
@@ -1474,12 +1462,13 @@ md_begin ()
   freeptr_static = (char *) malloc (PRIVATE_SIZE);	/* some private space please! */
 }
 
-
+#if 0 /* program exit will clean this up */
 void
 md_end ()
 {
   free (freeptr_static);
 }
+#endif
 
 /* Must be equal to MAX_PRECISON in atof-ieee.c */
 #define MAX_LITTLENUMS 6
@@ -1522,7 +1511,7 @@ md_atof (type, litP, sizeP)
       md_number_to_chars (litP, (long) (*--wordP), sizeof (LITTLENUM_TYPE));
       litP += sizeof (LITTLENUM_TYPE);
     }
-  return "";			/* Someone should teach Dean about null pointers */
+  return 0;
 }
 
 /* Convert number to chars in correct order */
@@ -1530,18 +1519,11 @@ md_atof (type, litP, sizeP)
 void
 md_number_to_chars (buf, value, nbytes)
      char *buf;
-     long value;
+     valueT value;
      int nbytes;
 {
-  while (nbytes--)
-    {
-#ifdef SHOW_NUM
-      printf ("%x ", value & 0xff);
-#endif
-      *buf++ = value;		/* Lint wants & MASK_CHAR. */
-      value >>= BITS_PER_CHAR;
-    }
-}				/* md_number_to_chars() */
+  number_to_chars_littleendian (buf, value, nbytes);
+}
 
 
 /* This is a variant of md_numbers_to_chars. The reason for its' existence
@@ -1720,7 +1702,7 @@ tc_aout_fix_to_chars (where, fixP, segment_address_in_file)
 		 ? S_GET_TYPE (fixP->fx_addsy)
 		 : fixP->fx_addsy->sy_number);
 
-  md_number_to_chars (where,
+  md_number_to_chars (where + 4,
 		      ((long) (r_symbolnum)
 		       | (long) (fixP->fx_pcrel << 24)
 		       | (long) (nbytes_r_length[fixP->fx_size] << 25)
@@ -1945,7 +1927,6 @@ md_estimate_size_before_relax (fragP, segment)
 			 (int) (fragP->fr_fix),
 			 4,
 			 fragP->fr_symbol,
-			 (symbolS *) 0,
 			 fragP->fr_offset,
 			 1,
 			 fragP->fr_pcrel_adjust,
@@ -1968,34 +1949,34 @@ md_estimate_size_before_relax (fragP, segment)
 
 int md_short_jump_size = 3;
 int md_long_jump_size = 5;
-int md_reloc_size = 8;		/* Size of relocation record */
+const int md_reloc_size = 8;	/* Size of relocation record */
 
 void
 md_create_short_jump (ptr, from_addr, to_addr, frag, to_symbol)
      char *ptr;
-     long from_addr, to_addr;
+     addressT from_addr, to_addr;
      fragS *frag;
      symbolS *to_symbol;
 {
-  long offset;
+  valueT offset;
 
   offset = to_addr - from_addr;
-  md_number_to_chars (ptr, (long) 0xEA, 1);
-  md_number_to_disp (ptr + 1, (long) offset, 2);
+  md_number_to_chars (ptr, (valueT) 0xEA, 1);
+  md_number_to_disp (ptr + 1, (valueT) offset, 2);
 }
 
 void
 md_create_long_jump (ptr, from_addr, to_addr, frag, to_symbol)
      char *ptr;
-     long from_addr, to_addr;
+     addressT from_addr, to_addr;
      fragS *frag;
      symbolS *to_symbol;
 {
-  long offset;
+  valueT offset;
 
   offset = to_addr - from_addr;
-  md_number_to_chars (ptr, (long) 0xEA, 2);
-  md_number_to_disp (ptr + 2, (long) offset, 4);
+  md_number_to_chars (ptr, (valueT) 0xEA, 2);
+  md_number_to_disp (ptr + 2, (valueT) offset, 4);
 }
 
 /* JF this is a new function to parse machine-dep options */
@@ -2066,13 +2047,12 @@ bit_fix_new (size, offset, min, max, add, base_type, base_adj)
 }
 
 void
-fix_new_ns32k (frag, where, size, add_symbol, sub_symbol, offset, pcrel,
+fix_new_ns32k (frag, where, size, add_symbol, offset, pcrel,
 	       pcrel_adjust, im_disp, bit_fixP, bsr)
      fragS *frag;		/* Which frag? */
      int where;			/* Where in that frag? */
      int size;			/* 1, 2  or 4 usually. */
      symbolS *add_symbol;	/* X_add_symbol. */
-     symbolS *sub_symbol;	/* X_subtract_symbol. */
      long offset;		/* X_add_number. */
      int pcrel;			/* TRUE if PC-relative relocation. */
      char pcrel_adjust;		/* not zero if adjustment of pcrel offset is needed */
@@ -2081,7 +2061,7 @@ fix_new_ns32k (frag, where, size, add_symbol, sub_symbol, offset, pcrel,
      char bsr;			/* sequent-linker-hack: 1 when relocobject is a bsr */
 
 {
-  fixS *fixP = fix_new (frag, where, size, add_symbol, sub_symbol,
+  fixS *fixP = fix_new (frag, where, size, add_symbol,
 			offset, pcrel, NO_RELOC);
 
   fixP->fx_pcrel_adjust = pcrel_adjust;
@@ -2090,6 +2070,40 @@ fix_new_ns32k (frag, where, size, add_symbol, sub_symbol, offset, pcrel,
   fixP->fx_bsr = bsr;
 }				/* fix_new_ns32k() */
 
+static void
+fix_new_ns32k_exp (frag, where, size, exp, pcrel,
+		   pcrel_adjust, im_disp, bit_fixP, bsr)
+     fragS *frag;		/* Which frag? */
+     int where;			/* Where in that frag? */
+     int size;			/* 1, 2  or 4 usually. */
+     expressionS *exp;		/* Expression. */
+     int pcrel;			/* TRUE if PC-relative relocation. */
+     char pcrel_adjust;		/* not zero if adjustment of pcrel offset is needed */
+     char im_disp;		/* true if the value to write is a displacement */
+     bit_fixS *bit_fixP;	/* pointer at struct of bit_fix's, ignored if NULL */
+     char bsr;			/* sequent-linker-hack: 1 when relocobject is a bsr */
+{
+  fixS *fixP = fix_new_exp (frag, where, size, exp, pcrel, NO_RELOC);
+
+  fixP->fx_pcrel_adjust = pcrel_adjust;
+  fixP->fx_im_disp = im_disp;
+  fixP->fx_bit_fixP = bit_fixP;
+  fixP->fx_bsr = bsr;
+}				/* fix_new_ns32k() */
+
+/* This is TC_CONS_FIX_NEW, called by emit_expr in read.c.  */
+
+void
+cons_fix_new_ns32k (frag, where, size, exp)
+     fragS *frag;		/* Which frag? */
+     int where;			/* Where in that frag? */
+     int size;			/* 1, 2  or 4 usually. */
+     expressionS *exp;		/* Expression. */
+{
+  fix_new_ns32k_exp (frag, where, size, exp,
+		     0, 0, 2, 0, 0);
+}
+
 /* We have no need to default values of symbols.  */
 
 symbolS *
@@ -2111,10 +2125,10 @@ md_operand (expressionP)
 }
 
 /* Round up a section size to the appropriate boundary.  */
-long
+valueT
 md_section_align (segment, size)
      segT segment;
-     long size;
+     valueT size;
 {
   return size;			/* Byte alignment is fine */
 }
