@@ -19,9 +19,9 @@
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
-  Written By Steve Chamberlain
-  sac@cygnus.com
-  */
+   Written By Steve Chamberlain
+   sac@cygnus.com
+ */
 
 #include <stdio.h>
 #include "as.h"
@@ -40,29 +40,26 @@ const char line_comment_chars[] = "!";
    pseudo-op name without dot
    function to call to execute this pseudo-op
    Integer arg to pass to the function
-   */
+ */
 
 void cons ();
-void s_align_bytes();
+void s_align_bytes ();
 
 const pseudo_typeS md_pseudo_table[] =
 {
   {"int", cons, 4},
-  {"data.b", cons, 1},
-  {"data.w", cons, 2},
-  {"data.l", cons, 4},
+  {"word", cons, 2},
   {"form", listing_psize, 0},
   {"heading", listing_title, 0},
   {"import", s_ignore, 0},
   {"page", listing_eject, 0},
   {"program", s_ignore, 0},
-/*  {"align", s_align_bytes, 0},*/
   {0, 0, 0}
 };
 
-/*int md_reloc_size;*/
+/*int md_reloc_size; */
 
-static int relax; /* set if -relax seen */
+static int relax;		/* set if -relax seen */
 
 const char EXP_CHARS[] = "eE";
 
@@ -73,7 +70,7 @@ const char FLT_CHARS[] = "rRsSfFdDxXpP";
 
 #define C(a,b) ENCODE_RELAX(a,b)
 
-#define JREG 14  	/* Register used as a temp when relaxing */
+#define JREG 14			/* Register used as a temp when relaxing */
 #define ENCODE_RELAX(what,length) (((what) << 4) + (length))
 #define GET_WHAT(x) ((x>>4))
 
@@ -96,14 +93,17 @@ const char FLT_CHARS[] = "rRsSfFdDxXpP";
 #define COND8_F 254
 #define COND8_M -256
 #define COND8_LENGTH 2
-#define COND12_F (4094 - 4) /* -4 since there are two extra */
-			      /* instructions needed */
+#define COND12_F (4094 - 4)	/* -4 since there are two extra */
+/* instructions needed */
 #define COND12_M -4096
 #define COND12_LENGTH 6
 #define COND32_F (1<<30)
 #define COND32_M -(1<<30)
 #define COND32_LENGTH 14
 
+#define COND8_RANGE(x) ((x) > COND8_M && (x) < COND8_F)
+#define COND12_RANGE(x) ((x) > COND12_M && (x) < COND12_F)
+
 #define UNCOND12_F 4094
 #define UNCOND12_M -4096
 #define UNCOND12_LENGTH 2
@@ -113,14 +113,14 @@ const char FLT_CHARS[] = "rRsSfFdDxXpP";
 #define UNCOND32_LENGTH 14
 
 
-relax_typeS md_relax_table[C (END, 0)];
+const relax_typeS md_relax_table[C (END, 0)];
 
 static struct hash_control *opcode_hash_control;	/* Opcode mnemonics */
 
 /*
-  This function is called once, at assembler startup time.  This should
-  set up all the tables, etc that the MD part of the assembler needs
-  */
+   This function is called once, at assembler startup time.  This should
+   set up all the tables, etc that the MD part of the assembler needs
+ */
 
 void
 md_begin ()
@@ -138,11 +138,12 @@ md_begin ()
 	  prev_name = opcode->name;
 	  hash_insert (opcode_hash_control, opcode->name, (char *) opcode);
 	}
-      else {
-	/* Make all the opcodes with the same name point to the same 
-	   string */
-	opcode->name = prev_name;
-      }
+      else
+	{
+	  /* Make all the opcodes with the same name point to the same
+	     string */
+	  opcode->name = prev_name;
+	}
     }
 
   /* Initialize the relax table */
@@ -180,10 +181,10 @@ static int reg_n;
 static expressionS immediate;	/* absolute expression */
 
 typedef struct
-{
-  sh_arg_type type;
-  int reg;
-}
+  {
+    sh_arg_type type;
+    int reg;
+  }
 
 sh_operand_info;
 
@@ -261,10 +262,24 @@ parse_reg (src, mode, reg)
 	}
     }
 
-
   return 0;
 }
 
+static symbolS *dot()
+{
+  const char *fake;
+
+  /* JF: '.' is pseudo symbol with value of current location
+     in current segment.  */
+  fake = FAKE_LABEL_NAME;
+  return  symbol_new (fake,
+		      now_seg,
+		      (valueT) frag_now_fix (),
+		      frag_now);
+
+}
+
+
 static
 char *
 parse_exp (s)
@@ -272,42 +287,24 @@ parse_exp (s)
 {
   char *save;
   char *new;
-  segT seg;
 
   save = input_line_pointer;
-
-
   input_line_pointer = s;
-
-  seg = expr (0, &immediate);
+  expression (&immediate);
+  if (immediate.X_op == O_absent)
+    as_bad ("missing operand");
   new = input_line_pointer;
   input_line_pointer = save;
-  if (SEG_NORMAL (seg))
-    return new;
-  switch (seg)
-    {
-    case SEG_ABSOLUTE:
-    case SEG_UNKNOWN:
-    case SEG_DIFFERENCE:
-    case SEG_BIG:
-    case SEG_REGISTER:
-      return new;
-    case SEG_ABSENT:
-      as_bad ("Missing operand");
-      return new;
-    default:
-      as_bad ("Don't understand operand of type %s", segment_name (seg));
-      return new;
-    }
+  return new;
 }
 
 
 /* The many forms of operand:
 
-   Rn			Register direct
-   @Rn			Register indirect
-   @Rn+			Autoincrement
-   @-Rn			Autodecrement
+   Rn                   Register direct
+   @Rn                  Register indirect
+   @Rn+                 Autoincrement
+   @-Rn                 Autodecrement
    @(disp:4,Rn)
    @(disp:8,GBR)
    @(disp:8,PC)
@@ -323,9 +320,10 @@ parse_exp (s)
  */
 
 static
-char *parse_at(src, op)
-char *src;
-sh_operand_info *op;
+char *
+parse_at (src, op)
+     char *src;
+     sh_operand_info *op;
 {
   int len;
   int mode;
@@ -334,7 +332,7 @@ sh_operand_info *op;
     {
       /* Must be predecrement */
       src++;
-      
+
       len = parse_reg (src, &mode, &(op->reg));
       if (mode != A_REG_N)
 	as_bad ("illegal register after @-");
@@ -345,7 +343,7 @@ sh_operand_info *op;
   else if (src[0] == '(')
     {
       /* Could be @(disp, rn), @(disp, gbr), @(disp, pc),  @(r0, gbr) or
-	 @(r0, rn) */
+         @(r0, rn) */
       src++;
       len = parse_reg (src, &mode, &(op->reg));
       if (len && mode == A_REG_N)
@@ -392,6 +390,11 @@ sh_operand_info *op;
 		}
 	      else if (mode == A_DISP_PC)
 		{
+		  /* Turn a plain @(4,pc) into @(.+4,pc) */
+		  if (immediate.X_op == O_constant) { 
+		    immediate.X_add_symbol = dot();
+		    immediate.X_op = O_symbol;
+		  }
 		  op->type = A_DISP_PC;
 		}
 	      else
@@ -405,10 +408,10 @@ sh_operand_info *op;
 	    }
 	}
       src += len;
-      if (src[0] != ')') 
-	as_bad("expecting )");
+      if (src[0] != ')')
+	as_bad ("expecting )");
       else
-	src ++;
+	src++;
     }
   else
     {
@@ -449,7 +452,7 @@ get_operand (ptr, op)
 
   else if (src[0] == '@')
     {
-      *ptr = parse_at(src, op);
+      *ptr = parse_at (src, op);
       return;
     }
   len = parse_reg (src, &mode, &(op->reg));
@@ -459,13 +462,13 @@ get_operand (ptr, op)
       op->type = mode;
       return;
     }
-  else 
+  else
     {
-    /* Not a reg, the only thing left is a displacement */
-    *ptr = parse_exp(src);
-    op->type = A_DISP_PC;
-    return;
-  }
+      /* Not a reg, the only thing left is a displacement */
+      *ptr = parse_exp (src);
+      op->type = A_DISP_PC;
+      return;
+    }
 }
 
 static
@@ -480,7 +483,7 @@ get_operands (info, args, operand)
   if (info->arg[0])
     {
       ptr++;
-      
+
       get_operand (&ptr, operand + 0);
       if (info->arg[1])
 	{
@@ -506,7 +509,7 @@ get_operands (info, args, operand)
 /* Passed a pointer to a list of opcodes which use different
    addressing modes, return the opcode which matches the opcodes
    provided
-   */
+ */
 
 static
 sh_opcode_info *
@@ -528,9 +531,9 @@ get_specific (opcode, operands)
 	  return 0;
 	}
       /* look at both operands needed by the opcodes and provided by
-	 the user - since an arg test will often fail on the same arg
-	 again and again, we'll try and test the last failing arg the
-	 first on each opcode try */
+         the user - since an arg test will often fail on the same arg
+         again and again, we'll try and test the last failing arg the
+         first on each opcode try */
 
       for (n = 0; this_try->arg[n]; n++)
 	{
@@ -545,13 +548,20 @@ get_specific (opcode, operands)
 	    case A_DISP_PC:
 	    case A_MACH:
 	    case A_PR:
-	    case A_MACL:	      
-	      if (user->type != arg) goto fail;
+	    case A_MACL:
+	      if (user->type != arg)
+		goto fail;
 	      break;
 	    case A_R0:
 	      /* opcode needs r0 */
-	      if (user->type != A_REG_N || user->reg != 0) goto fail;
-		break;
+	      if (user->type != A_REG_N || user->reg != 0)
+		goto fail;
+	      break;
+	    case A_R0_GBR:
+	      if (user->type != A_R0_GBR || user->reg != 0)
+		goto fail;
+	      break;
+
 	    case A_REG_N:
 	    case A_INC_N:
 	    case A_DEC_N:
@@ -559,9 +569,16 @@ get_specific (opcode, operands)
 	    case A_IND_R0_REG_N:
 	    case A_DISP_REG_N:
 	      /* Opcode needs rn */
-	      if (user->type != arg) goto fail;
+	      if (user->type != arg)
+		goto fail;
 	      reg_n = user->reg;
 	      break;
+	    case A_GBR:
+	    case A_SR:
+	    case A_VBR:
+	      if (user->type != arg)
+		goto fail;
+	      break;
 
 	    case A_REG_M:
 	    case A_INC_M:
@@ -570,7 +587,8 @@ get_specific (opcode, operands)
 	    case A_IND_R0_REG_M:
 	    case A_DISP_REG_M:
 	      /* Opcode needs rn */
-	      if (user->type != arg  - A_REG_M + A_REG_N) goto fail;
+	      if (user->type != arg - A_REG_M + A_REG_N)
+		goto fail;
 	      reg_m = user->reg;
 	      break;
 	    default:
@@ -593,7 +611,7 @@ check (operand, low, high)
      int low;
      int high;
 {
-  if (operand->X_seg != SEG_ABSOLUTE
+  if (operand->X_op != O_constant
       || operand->X_add_number < low
       || operand->X_add_number > high)
     {
@@ -609,47 +627,46 @@ insert (where, how, pcrel)
      int how;
      int pcrel;
 {
-  fix_new (frag_now,
-	   where - frag_now->fr_literal,
-	   4,
-	   immediate.X_add_symbol,
-	   immediate.X_subtract_symbol,
-	   immediate.X_add_number,
-	   pcrel,
-	   how);
+  fix_new_exp (frag_now,
+	       where - frag_now->fr_literal,
+	       4,
+	       &immediate,
+	       pcrel,
+	       how);
 
 }
 
 static void
 build_relax (opcode)
-sh_opcode_info *opcode;
+     sh_opcode_info *opcode;
 {
   int len;
   char *p;
-  if (opcode->arg[0] == A_BDISP8) 
+  if (opcode->arg[0] == A_BDISP8)
     {
-      p = frag_var(rs_machine_dependent,
-		   md_relax_table[C(COND_JUMP, COND32)].rlx_length,
-		   len = md_relax_table[C(COND_JUMP,  COND8)].rlx_length,
-		   C(COND_JUMP, 0),
-		   immediate.X_add_symbol,
-		   immediate.X_add_number,
-		   0);
+      p = frag_var (rs_machine_dependent,
+		    md_relax_table[C (COND_JUMP, COND32)].rlx_length,
+		    len = md_relax_table[C (COND_JUMP, COND8)].rlx_length,
+		    C (COND_JUMP, 0),
+		    immediate.X_add_symbol,
+		    immediate.X_add_number,
+		    0);
       p[0] = (opcode->nibbles[0] << 4) | (opcode->nibbles[1]);
     }
-  else if (opcode->arg[0] == A_BDISP12) 
+  else if (opcode->arg[0] == A_BDISP12)
     {
-      p = frag_var(rs_machine_dependent,
-		   md_relax_table[C(UNCOND_JUMP, UNCOND32)].rlx_length,
-		   len = md_relax_table[C(UNCOND_JUMP,  UNCOND12)].rlx_length,
-		   C(UNCOND_JUMP, 0),
-		   immediate.X_add_symbol,
-		   immediate.X_add_number,
-		   0);
-      p[0] = (opcode->nibbles[0] << 4) ;
+      p = frag_var (rs_machine_dependent,
+		    md_relax_table[C (UNCOND_JUMP, UNCOND32)].rlx_length,
+		 len = md_relax_table[C (UNCOND_JUMP, UNCOND12)].rlx_length,
+		    C (UNCOND_JUMP, 0),
+		    immediate.X_add_symbol,
+		    immediate.X_add_number,
+		    0);
+      p[0] = (opcode->nibbles[0] << 4);
     }
 
 }
+
 /* Now we know what sort of opcodes it is, lets build the bytes -
  */
 static void
@@ -685,31 +702,31 @@ build_Mytes (opcode, operand)
 	      nbuf[index] = reg_m;
 	      break;
 	    case DISP_4:
-	      insert(output + 1, R_SH_IMM4,0);
+	      insert (output + 1, R_SH_IMM4, 0);
 	      break;
 	    case IMM_4BY4:
-	      insert(output + 1, R_SH_IMM4BY4,0);
+	      insert (output + 1, R_SH_IMM4BY4, 0);
 	      break;
 	    case IMM_4BY2:
-	      insert(output + 1, R_SH_IMM4BY2,0);
+	      insert (output + 1, R_SH_IMM4BY2, 0);
 	      break;
 	    case IMM_4:
-	      insert(output + 1, R_SH_IMM4,0);
+	      insert (output + 1, R_SH_IMM4, 0);
 	      break;
 	    case IMM_8BY4:
-	      insert(output + 1, R_SH_IMM8BY4,0);
+	      insert (output + 1, R_SH_IMM8BY4, 0);
 	      break;
 	    case IMM_8BY2:
-	      insert(output + 1, R_SH_IMM8BY2,0);
+	      insert (output + 1, R_SH_IMM8BY2, 0);
 	      break;
 	    case IMM_8:
-	      insert(output + 1, R_SH_IMM8,0);
+	      insert (output + 1, R_SH_IMM8, 0);
 	      break;
 	    case PCRELIMM_8BY4:
-	      insert(output + 1, R_SH_PCRELIMM8BY4,0);
+	      insert (output + 1, R_SH_PCRELIMM8BY4, 1);
 	      break;
 	    case PCRELIMM_8BY2:
-	      insert(output + 1, R_SH_PCRELIMM8BY2,0);
+	      insert (output + 1, R_SH_PCRELIMM8BY2, 1);
 	      break;
 	    default:
 	      printf ("failed for %d\n", i);
@@ -723,35 +740,36 @@ build_Mytes (opcode, operand)
 /* This is the guts of the machine-dependent assembler.  STR points to a
    machine dependent instruction.  This function is supposed to emit
    the frags/bytes it assembles to.
-   */
+ */
 
 void
-md_assemble(str)
- char *str;
+md_assemble (str)
+     char *str;
 {
-  unsigned  char *op_start;
-  unsigned  char *op_end;
+  unsigned char *op_start;
+  unsigned char *op_end;
   sh_operand_info operand[2];
   sh_opcode_info *opcode;
-  unsigned  char *name;
-
+  char name[20];
   int nlen = 0;
-
+  char *p;
   /* Drop leading whitespace */
   while (*str == ' ')
     str++;
 
   /* find the op code end */
-  for (name = op_start = op_end = (unsigned char *)(str);
-       *op_end &&
-       !is_end_of_line[*op_end] && *op_end != ' ';
+  for (op_start = op_end = (unsigned char *) (str);
+       *op_end
+       && nlen < 20
+       && !is_end_of_line[*op_end] && *op_end != ' ';
        op_end++)
     {
+      name[nlen] = op_start[nlen];
       nlen++;
     }
   name[nlen] = 0;
 
-  if (op_end == op_start)
+  if (nlen == 0)
     {
       as_bad ("can't find opcode ");
     }
@@ -765,15 +783,17 @@ md_assemble(str)
     }
 
   if (opcode->arg[0] == A_BDISP12
-      || opcode->arg[0] == A_BDISP8) 
+      || opcode->arg[0] == A_BDISP8)
     {
-      input_line_pointer = parse_exp(op_end + 1);
-      build_relax(opcode);
+      parse_exp (op_end + 1);
+      build_relax (opcode);
     }
-  else 
+  else
     {
-      input_line_pointer = get_operands (opcode, op_end, operand);
-
+      if (opcode->arg[0] != A_END)
+	{
+	  get_operands (opcode, op_end, operand);
+	}
       opcode = get_specific (opcode, operand);
 
       if (opcode == 0)
@@ -813,11 +833,6 @@ DEFUN (tc_headers_hook, (headers),
   printf ("call to tc_headers_hook \n");
 }
 
-void
-DEFUN_VOID (md_end)
-{
-}
-
 /* Various routines to kill one day */
 /* Equal to MAX_PRECISION in atof-ieee.c */
 #define MAX_LITTLENUMS 6
@@ -825,7 +840,7 @@ DEFUN_VOID (md_end)
 /* Turn a string in input_line_pointer into a floating point constant of type
    type, and store the appropriate bytes in *litP.  The number of LITTLENUMS
    emitted is stored in *sizeP .  An error message is returned, or NULL on OK.
-   */
+ */
 char *
 md_atof (type, litP, sizeP)
      char type;
@@ -878,7 +893,7 @@ md_atof (type, litP, sizeP)
       md_number_to_chars (litP, (long) (*wordP++), sizeof (LITTLENUM_TYPE));
       litP += sizeof (LITTLENUM_TYPE);
     }
-  return "";			/* Someone should teach Dean about null pointers */
+  return 0;
 }
 
 int
@@ -891,7 +906,7 @@ md_parse_option (argP, cntP, vecP)
   if (!strcmp (*argP, "relax"))
     {
       relax = 1;
-      **argP =0;
+      **argP = 0;
     }
   return 1;
 }
@@ -908,8 +923,8 @@ tc_Nout_fix_to_chars ()
 void
 md_create_short_jump (ptr, from_Nddr, to_Nddr, frag, to_symbol)
      char *ptr;
-     long from_Nddr;
-     long to_Nddr;
+     addressT from_Nddr;
+     addressT to_Nddr;
      fragS *frag;
      symbolS *to_symbol;
 {
@@ -919,7 +934,7 @@ md_create_short_jump (ptr, from_Nddr, to_Nddr, frag, to_symbol)
 void
 md_create_long_jump (ptr, from_Nddr, to_Nddr, frag, to_symbol)
      char *ptr;
-     long from_Nddr, to_Nddr;
+     addressT from_Nddr, to_Nddr;
      fragS *frag;
      symbolS *to_symbol;
 {
@@ -927,17 +942,17 @@ md_create_long_jump (ptr, from_Nddr, to_Nddr, frag, to_symbol)
 }
 
 /*
-called after relaxing, change the frags so they know how big they are
-*/
+   called after relaxing, change the frags so they know how big they are
+ */
 void
 md_convert_frag (headers, fragP)
      object_headers *headers;
      fragS *fragP;
 
 {
-  unsigned  char *buffer = (unsigned char *)(fragP->fr_fix + fragP->fr_literal);
+  unsigned char *buffer = (unsigned char *) (fragP->fr_fix + fragP->fr_literal);
   int donerelax = 0;
-  int targ_addr = (S_GET_VALUE (fragP->fr_symbol) + fragP->fr_offset);  
+  int targ_addr = ((fragP->fr_symbol ? S_GET_VALUE (fragP->fr_symbol) : 0) + fragP->fr_offset);
   switch (fragP->fr_subtype)
     {
     case C (COND_JUMP, COND8):
@@ -945,9 +960,10 @@ md_convert_frag (headers, fragP)
 	/* Get the address of the end of the instruction */
 	int next_inst = fragP->fr_fix + fragP->fr_address + 2;
 
-	int disp = targ_addr - next_inst -2 ;
+	int disp = targ_addr - next_inst - 2;
 	disp /= 2;
-	md_number_to_chars (buffer + 1, disp, 1);
+	
+	md_number_to_chars (buffer + 1, disp & 0xff, 1);
 	fragP->fr_fix += 2;
 	fragP->fr_var = 0;
       }
@@ -959,11 +975,11 @@ md_convert_frag (headers, fragP)
 	int next_inst = fragP->fr_fix + fragP->fr_address + 2;
 
 	int t;
-	int disp = targ_addr - next_inst -2 ;
+	int disp = targ_addr - next_inst - 2;
 
 	disp /= 2;
 	t = buffer[0] & 0xf0;
-	md_number_to_chars (buffer , disp, 2);
+	md_number_to_chars (buffer, disp & 0xfff, 2);
 	buffer[0] = (buffer[0] & 0xf) | t;
 	fragP->fr_fix += 2;
 	fragP->fr_var = 0;
@@ -978,45 +994,44 @@ md_convert_frag (headers, fragP)
 	   mov.w @(0, PC), r14
 	   .long disp
 	   foo: bra @r14
-	   */
-	
-	int next_inst =
-	  fragP->fr_fix + fragP->fr_address + UNCOND32_LENGTH;
+	 */
 
-	int disp = targ_addr - next_inst  ;
+	int next_inst =
+	fragP->fr_fix + fragP->fr_address + UNCOND32_LENGTH;
+
+	int disp = targ_addr - next_inst;
 	int t = buffer[0] & 0x10;
 
 	disp /= 2;
 
 	buffer[0] = 0xa0;	/* branch over move and disp */
 	buffer[1] = 3;
-	buffer[2] = 0xd0 | JREG; /* Build mov insn */
+	buffer[2] = 0xd0 | JREG;	/* Build mov insn */
 	buffer[3] = 0x00;
 
 	buffer[4] = 0;		/* space for 32 bit jump disp */
-	buffer[5] = 0; 
+	buffer[5] = 0;
 	buffer[6] = 0;
 	buffer[7] = 0;
 
-	buffer[10] = 0x40 | JREG; /* Build jmp @JREG */
-	buffer[11] = t ? 0xb : 0x2b ;
+	buffer[10] = 0x40 | JREG;	/* Build jmp @JREG */
+	buffer[11] = t ? 0xb : 0x2b;
 
 	buffer[12] = 0x20;	/* build nop */
 	buffer[13] = 0x0b;
 
 	/* Make reloc for the long disp */
-	fix_new(fragP, 
-		fragP->fr_fix + 4,
-		4,
-		fragP->fr_symbol,
-		0,
-		fragP->fr_offset, 
-		0,
-		R_SH_IMM32);
+	fix_new (fragP,
+		 fragP->fr_fix + 4,
+		 4,
+		 fragP->fr_symbol,
+		 fragP->fr_offset,
+		 0,
+		 R_SH_IMM32);
 	fragP->fr_fix += UNCOND32_LENGTH;
 	fragP->fr_var = 0;
 	donerelax = 1;
-	
+
       }
       break;
 
@@ -1024,16 +1039,16 @@ md_convert_frag (headers, fragP)
       {
 	/* A bcond won't fit, so turn it into a b!cond; bra disp; nop */
 	int next_inst =
-	  fragP->fr_fix + fragP->fr_address + 6;
+	fragP->fr_fix + fragP->fr_address + 6;
 
-	int disp = targ_addr - next_inst  ;
+	int disp = targ_addr - next_inst;
 	disp /= 2;
-	md_number_to_chars (buffer + 2, disp, 2);
+	md_number_to_chars (buffer + 2, disp & 0xfff, 2);
 	buffer[0] ^= 0x2;	/* Toggle T/F bit */
 	buffer[1] = 1;		/* branch over jump and nop */
-	buffer[2] = (buffer[2] & 0xf) | 0xa0; /* Build jump insn */
+	buffer[2] = (buffer[2] & 0xf) | 0xa0;	/* Build jump insn */
 	buffer[4] = 0x20;	/* Build nop */
-	buffer[5] = 0x0b; 
+	buffer[5] = 0x0b;
 	fragP->fr_fix += 6;
 	fragP->fr_var = 0;
 	donerelax = 1;
@@ -1043,46 +1058,45 @@ md_convert_frag (headers, fragP)
     case C (COND_JUMP, COND32):
     case C (COND_JUMP, UNDEF_WORD_DISP):
       {
-	/* A bcond won't fit and it won't go into a 12 bit 
+	/* A bcond won't fit and it won't go into a 12 bit
 	   displacement either, the code sequence looks like:
 	   b!cond foop
 	   mov.w @(n, PC), r14
-	   jmp	@r14
+	   jmp  @r14
 	   nop
 	   .long where
-	   foop:	   
-	   */
-	
-	int next_inst =
-	  fragP->fr_fix + fragP->fr_address + COND32_LENGTH;
+	   foop:
+	 */
 
-	int disp = targ_addr - next_inst  ;
+	int next_inst =
+	fragP->fr_fix + fragP->fr_address + COND32_LENGTH;
+
+	int disp = targ_addr - next_inst;
 	disp /= 2;
 
 	buffer[0] ^= 0x2;	/* Toggle T/F bit */
 #define JREG 14
 	buffer[1] = 5;		/* branch over mov, jump, nop and ptr */
-	buffer[2] = 0xd0 | JREG; /* Build mov insn */
+	buffer[2] = 0xd0 | JREG;	/* Build mov insn */
 	buffer[3] = 0x2;
-	buffer[4] = 0x40 | JREG; /* Build jmp @JREG */
-	buffer[5] = 0x0b; 
+	buffer[4] = 0x40 | JREG;	/* Build jmp @JREG */
+	buffer[5] = 0x0b;
 	buffer[6] = 0x20;	/* build nop */
 	buffer[7] = 0x0b;
 	buffer[8] = 0;		/* space for 32 bit jump disp */
-	buffer[9] = 0; 
+	buffer[9] = 0;
 	buffer[10] = 0;
 	buffer[11] = 0;
 	buffer[12] = 0;
 	buffer[13] = 0;
 	/* Make reloc for the long disp */
-	fix_new(fragP, 
-		fragP->fr_fix + 8,
-		4,
-		fragP->fr_symbol,
-		0,
-		fragP->fr_offset, 
-		0,
-		R_SH_IMM32);
+	fix_new (fragP,
+		 fragP->fr_fix + 8,
+		 4,
+		 fragP->fr_symbol,
+		 fragP->fr_offset,
+		 0,
+		 R_SH_IMM32);
 	fragP->fr_fix += COND32_LENGTH;
 	fragP->fr_var = 0;
 	donerelax = 1;
@@ -1093,19 +1107,20 @@ md_convert_frag (headers, fragP)
       abort ();
     }
 
-  if (donerelax && !relax) 
+  if (donerelax && !relax)
     {
-      as_bad("Offset doesn't fit at 0x%x, trying to get to 0x%x",
-	     fragP->fr_address,
-	     targ_addr);
+      as_warn ("Offset doesn't fit at 0x%x, trying to get to %s+0x%x",
+	       fragP->fr_address,
+	       fragP->fr_symbol  ?    S_GET_NAME(fragP->fr_symbol): "",
+	       targ_addr);
     }
 
 }
 
-long
+valueT
 DEFUN (md_section_align, (seg, size),
        segT seg AND
-       long size)
+       valueT size)
 {
   return ((size + (1 << section_alignment[(int) seg]) - 1)
 	  & (-1 << section_alignment[(int) seg]));
@@ -1121,30 +1136,33 @@ md_apply_fix (fixP, val)
   int addr = fixP->fx_frag->fr_address + fixP->fx_where;
   if (fixP->fx_r_type == 0)
     {
-      fixP->fx_r_type = R_SH_IMM32 ;
+      if (fixP->fx_size == 2)
+	fixP->fx_r_type = R_SH_IMM16;
+      else
+	fixP->fx_r_type = R_SH_IMM32;
     }
 
   switch (fixP->fx_r_type)
     {
-      
+
     case R_SH_IMM4:
       *buf = (*buf & 0xf0) | (val & 0xf);
       break;
 
     case R_SH_IMM4BY2:
-      *buf = (*buf & 0xf0) | ((val>>1) & 0xf);
+      *buf = (*buf & 0xf0) | ((val >> 1) & 0xf);
       break;
 
     case R_SH_IMM4BY4:
-      *buf = (*buf & 0xf0) | ((val>>2) & 0xf);
+      *buf = (*buf & 0xf0) | ((val >> 2) & 0xf);
       break;
 
     case R_SH_IMM8BY2:
-      *buf = val >>1;
+      *buf = val >> 1;
       break;
 
     case R_SH_IMM8BY4:
-      *buf = val >>2;
+      *buf = val >> 2;
       break;
 
     case R_SH_IMM8:
@@ -1153,35 +1171,41 @@ md_apply_fix (fixP, val)
 
     case R_SH_PCRELIMM8BY4:
       addr &= ~1;
-      
+#if 0
       if (val & 0x3)
-	as_warn("non aligned displacement at %x\n", addr);
-      val -=(addr+4);
-      val+=3;
+	as_warn ("non aligned displacement at %x\n", addr);
+#endif
+      /*      val -= (addr + 4); */
+      val += 3;
       val /= 4;
       if (val & ~0xff)
-	as_warn("pcrel too far at %x\n", addr);
-      
-      *buf = val ;
+	as_warn ("pcrel too far at %x\n", addr);
+
+      *buf = val;
       break;
 
     case R_SH_PCRELIMM8BY2:
       addr &= ~1;
       if (val & 0x1)
-	as_bad("odd displacement at %x\n", addr);
-      val -= (addr+4);
+	as_bad ("odd displacement at %x\n", addr);
+      /*      val -= (addr + 4); */
       val++;
       val /= 2;
       if (val & ~0xff)
-	as_warn("pcrel too far at %x\n", addr);
-      *buf= val;
+	as_warn ("pcrel too far at %x\n", addr);
+      *buf = val;
       break;
 
     case R_SH_IMM32:
-      *buf++ = val >>24;
-      *buf++ = val >>16;
-      *buf++ = val >>8;
-      *buf++ = val >>0;
+      *buf++ = val >> 24;
+      *buf++ = val >> 16;
+      *buf++ = val >> 8;
+      *buf++ = val >> 0;
+      break;
+
+    case R_SH_IMM16:
+      *buf++ = val >> 8;
+      *buf++ = val >> 0;
       break;
 
     default:
@@ -1197,9 +1221,9 @@ DEFUN (md_operand, (expressionP), expressionS * expressionP)
 int md_long_jump_size;
 
 /*
-called just before address relaxation, return the length
-by which a fragment must grow to reach it's destination
-*/
+   called just before address relaxation, return the length
+   by which a fragment must grow to reach it's destination
+ */
 int
 md_estimate_size_before_relax (fragP, segment_type)
      register fragS *fragP;
@@ -1207,9 +1231,14 @@ md_estimate_size_before_relax (fragP, segment_type)
 {
   switch (fragP->fr_subtype)
     {
-    case C(UNCOND_JUMP, UNDEF_DISP):
+    case C (UNCOND_JUMP, UNDEF_DISP):
       /* used to be a branch to somewhere which was unknown */
-      if (S_GET_SEGMENT (fragP->fr_symbol) == segment_type)
+      if (!fragP->fr_symbol)
+	{
+	  fragP->fr_subtype = C (UNCOND_JUMP, UNCOND12);
+	  fragP->fr_var = md_relax_table[C (UNCOND_JUMP, UNCOND12)].rlx_length;
+	}
+      else if (S_GET_SEGMENT (fragP->fr_symbol) == segment_type)
 	{
 	  fragP->fr_subtype = C (UNCOND_JUMP, UNCOND12);
 	  fragP->fr_var = md_relax_table[C (UNCOND_JUMP, UNCOND12)].rlx_length;
@@ -1220,27 +1249,35 @@ md_estimate_size_before_relax (fragP, segment_type)
 	  fragP->fr_var = md_relax_table[C (UNCOND_JUMP, UNCOND32)].rlx_length;
 	  return md_relax_table[C (UNCOND_JUMP, UNCOND32)].rlx_length;
 	}
-      break; 
-      
+      break;
+
     default:
       abort ();
     case C (COND_JUMP, UNDEF_DISP):
       /* used to be a branch to somewhere which was unknown */
-      if (S_GET_SEGMENT (fragP->fr_symbol) == segment_type)
+      if (fragP->fr_symbol
+	  && S_GET_SEGMENT (fragP->fr_symbol) == segment_type)
 	{
 	  /* Got a symbol and it's defined in this segment, become byte
-	 sized - maybe it will fix up */
+	     sized - maybe it will fix up */
 	  fragP->fr_subtype = C (COND_JUMP, COND8);
 	  fragP->fr_var = md_relax_table[C (COND_JUMP, COND8)].rlx_length;
 	}
-      else
+      else if (fragP->fr_symbol)
 	{
 	  /* Its got a segment, but its not ours, so it will always be long */
 	  fragP->fr_subtype = C (COND_JUMP, UNDEF_WORD_DISP);
 	  fragP->fr_var = md_relax_table[C (COND_JUMP, COND32)].rlx_length;
 	  return md_relax_table[C (COND_JUMP, COND32)].rlx_length;
 	}
-      break; 
+      else
+	{
+	  /* We know the abs value */
+	  fragP->fr_subtype = C (COND_JUMP, COND8);
+	  fragP->fr_var = md_relax_table[C (COND_JUMP, COND8)].rlx_length;
+	}
+
+      break;
     }
   return fragP->fr_var;
 }
@@ -1250,32 +1287,18 @@ md_estimate_size_before_relax (fragP, segment_type)
 void
 md_number_to_chars (ptr, use, nbytes)
      char *ptr;
-     long use;
+     valueT use;
      int nbytes;
 {
-  switch (nbytes)
-    {
-    case 4:
-      *ptr++ = (use >> 24) & 0xff;
-    case 3:
-      *ptr++ = (use >> 16) & 0xff;
-    case 2:
-      *ptr++ = (use >> 8) & 0xff;
-    case 1:
-      *ptr++ = (use >> 0) & 0xff;
-      break;
-    default:
-      abort ();
-    }
+  number_to_chars_bigendian (ptr, use, nbytes);
 }
+
 long
 md_pcrel_from (fixP)
      fixS *fixP;
-
 {
-  int gap = fixP->fx_size + fixP->fx_where +
-    fixP->fx_frag->fr_address;
-  return gap;  
+  int gap = fixP->fx_size + fixP->fx_where +  fixP->fx_frag->fr_address - 1 ;
+  return gap;
 }
 
 void
@@ -1350,9 +1373,8 @@ tc_reloc_mangle (fix_ptr, intr, base)
 }
 
 int
-tc_coff_sizemachdep(frag)
-fragS *frag;
+tc_coff_sizemachdep (frag)
+     fragS *frag;
 {
   return md_relax_table[frag->fr_subtype].rlx_length;
-}  
-/* end of tc-sh.c */
+}
