@@ -23,21 +23,12 @@
 #include <ctype.h>
 
 #include "as.h"
+#include "subsegs.h"
 
 /* careful, this file includes data *declarations* */
 #include "opcode/sparc.h"
 
-void md_begin ();
-void md_end ();
-void md_number_to_chars ();
-void md_assemble ();
-char *md_atof ();
-void md_create_short_jump ();
-void md_create_long_jump ();
-int md_estimate_size_before_relax ();
-void md_ri_to_chars ();
-symbolS *md_undefined_symbol ();
-static void sparc_ip ();
+static void sparc_ip PARAMS ((char *));
 
 static enum sparc_architecture current_architecture = v6;
 static int architecture_requested;
@@ -50,24 +41,11 @@ const relax_typeS md_relax_table[1];
 /* handle of the OPCODE hash table */
 static struct hash_control *op_hash = NULL;
 
-static void s_seg (), s_proc (), s_data1 (), s_reserve (), s_common ();
-extern void s_globl (), s_long (), s_short (), s_space (), cons ();
-extern void s_align_bytes (), s_ignore ();
-
-/* Ugly hack to keep non-BFD version working.  */
-#ifndef BFD_ASSEMBLER
-#define BFD_RELOC_NONE		NO_RELOC
-#define BFD_RELOC_32		RELOC_32
-#define BFD_RELOC_HI22		RELOC_HI22
-#define BFD_RELOC_LO10		RELOC_LO10
-#define BFD_RELOC_SPARC_WDISP22	RELOC_WDISP22
-#define BFD_RELOC_32_PCREL_S2	RELOC_WDISP30
-#define BFD_RELOC_SPARC22	RELOC_22
-#define BFD_RELOC_SPARC_BASE13	RELOC_BASE13
-#define BFD_RELOC_SPARC13	RELOC_13
-#define BFD_RELOC_SPARC_BASE22	RELOC_BASE22
-#define subseg_set		subseg_new
-#endif
+static void s_data1 PARAMS ((void));
+static void s_seg PARAMS ((int));
+static void s_proc PARAMS ((int));
+static void s_reserve PARAMS ((int));
+static void s_common PARAMS ((int));
 
 const pseudo_typeS md_pseudo_table[] =
 {
@@ -81,6 +59,13 @@ const pseudo_typeS md_pseudo_table[] =
   {"seg", s_seg, 0},
   {"skip", s_space, 0},
   {"word", cons, 4},
+#ifdef OBJ_ELF
+  /* these are specific to sparc/svr4 */
+  {"pushsection", obj_elf_section, 0},
+  {"popsection", obj_elf_previous, 0},
+  {"uaword", cons, 4},
+  {"uahalf", cons, 2},
+#endif
   {NULL, 0, 0},
 };
 
@@ -113,12 +98,11 @@ const char EXP_CHARS[] = "eE";
 const char FLT_CHARS[] = "rRsSfFdDxXpP";
 
 /* Also be aware that MAXIMUM_NUMBER_OF_CHARS_FOR_FLOAT may have to be
-   changed in read.c .  Ideally it shouldn't have to know about it at all,
-   but nothing is ideal around here.
-   */
+   changed in read.c.  Ideally it shouldn't have to know about it at all,
+   but nothing is ideal around here.  */
 
 static unsigned char octal[256];
-#define isoctal(c)  octal[c]
+#define isoctal(c)  octal[(unsigned char) (c)]
 static unsigned char toHex[256];
 
 struct sparc_it
@@ -128,15 +112,24 @@ struct sparc_it
     struct nlist *nlistp;
     expressionS exp;
     int pcrel;
-#ifdef BFD_ASSEMBLER
     bfd_reloc_code_real_type reloc;
-#else
-    enum reloc_type reloc;
-#endif
   };
 
 struct sparc_it the_insn, set_insn;
 
+static INLINE int
+in_signed_range (val, max)
+     bfd_signed_vma val, max;
+{
+  if (max <= 0)
+    abort ();
+  if (val > max)
+    return 0;
+  if (val < ~max)
+    return 0;
+  return 1;
+}
+
 #if 0
 static void print_insn PARAMS ((struct sparc_it *insn));
 #endif
@@ -156,14 +149,14 @@ static int special_case;
  * sort of like s_lcomm
  *
  */
+#ifndef OBJ_ELF
 static int max_alignment = 15;
+#endif
 
 static void
-s_reserve ()
+s_reserve (ignore)
+     int ignore;
 {
-#ifndef OBJ_AOUT
-  as_fatal ("s_reserve only defined for a.out");
-#else
   char *name;
   char *p;
   char c;
@@ -198,13 +191,17 @@ s_reserve ()
   symbolP = symbol_find_or_make (name);
   *p = c;
 
-  if (strncmp (input_line_pointer, ",\"bss\"", 6) != 0)
+  if (strncmp (input_line_pointer, ",\"bss\"", 6) != 0
+      && strncmp (input_line_pointer, ",\".bss\"", 7) != 0)
     {
-      as_bad ("bad .reserve segment: `%s'", input_line_pointer);
+      as_bad ("bad .reserve segment -- expected BSS segment", input_line_pointer);
       return;
-    }				/* if not bss */
+    }
 
-  input_line_pointer += 6;
+  if (input_line_pointer[2] == '.')
+    input_line_pointer += 7;
+  else
+    input_line_pointer += 6;
   SKIP_WHITESPACE ();
 
   if (*input_line_pointer == ',')
@@ -219,12 +216,14 @@ s_reserve ()
 	}
 
       align = get_absolute_expression ();
+#ifndef OBJ_ELF
       if (align > max_alignment)
 	{
 	  align = max_alignment;
 	  as_warn ("Alignment too large: %d. assumed.", align);
 	}
-      else if (align < 0)
+#endif
+      if (align < 0)
 	{
 	  align = 0;
 	  as_warn ("Alignment negative. 0 assumed.");
@@ -247,14 +246,17 @@ s_reserve ()
   else
     align = 0;
 
-  if (S_GET_OTHER (symbolP) == 0
+  if ((S_GET_SEGMENT (symbolP) == bss_section
+       || !S_IS_DEFINED (symbolP))
+#ifdef OBJ_AOUT
+      && S_GET_OTHER (symbolP) == 0
       && S_GET_DESC (symbolP) == 0
-      && (S_GET_SEGMENT (symbolP) == bss_section
-	  || !S_IS_DEFINED (symbolP)))
+#endif
+      )
     {
       if (! need_pass_2)
 	{
-	  char *p;
+	  char *pfrag;
 	  segT current_seg = now_seg;
 	  subsegT current_subseg = now_subseg;
 
@@ -268,9 +270,9 @@ s_reserve ()
 	    symbolP->sy_frag->fr_symbol = NULL;
 
 	  symbolP->sy_frag = frag_now;
-	  p = frag_var (rs_org, 1, 1, (relax_substateT)0, symbolP,
-			size, (char *)0);
-	  *p = 0;
+	  pfrag = frag_var (rs_org, 1, 1, (relax_substateT)0, symbolP,
+			    size, (char *)0);
+	  *pfrag = 0;
 
 	  S_SET_SEGMENT (symbolP, bss_section);
 
@@ -283,17 +285,17 @@ s_reserve ()
     }				/* if not redefining */
 
   demand_empty_rest_of_line ();
-#endif
 }
 
 static void
-s_common ()
+s_common (ignore)
+     int ignore;
 {
-  register char *name;
-  register char c;
-  register char *p;
-  register int temp;
-  register symbolS *symbolP;
+  char *name;
+  char c;
+  char *p;
+  int temp, size;
+  symbolS *symbolP;
 
   name = input_line_pointer;
   c = get_symbol_end ();
@@ -314,6 +316,7 @@ s_common ()
       ignore_rest_of_line ();
       return;
     }
+  size = temp;
   *p = 0;
   symbolP = symbol_find_or_make (name);
   *p = c;
@@ -325,76 +328,131 @@ s_common ()
     }
   if (S_GET_VALUE (symbolP) != 0)
     {
-      if (S_GET_VALUE (symbolP) != temp)
+      if (S_GET_VALUE (symbolP) != size)
 	{
-	  as_warn ("Length of .comm \"%s\" is already %d. Not changed to %d.",
-		   S_GET_NAME (symbolP), S_GET_VALUE (symbolP), temp);
+	  as_warn ("Length of .comm \"%s\" is already %ld. Not changed to %d.",
+		   S_GET_NAME (symbolP), (long) S_GET_VALUE (symbolP), size);
 	}
     }
   else
     {
-      S_SET_VALUE (symbolP, temp);
+#ifndef OBJ_ELF
+      S_SET_VALUE (symbolP, (valueT) size);
       S_SET_EXTERNAL (symbolP);
-#ifdef OBJ_ELF
-      /* But not for a.out ... find some sensible way to characterize
-	 this.  */
-      S_SET_SEGMENT (symbolP, now_seg);
 #endif
     }
   know (symbolP->sy_frag == &zero_address_frag);
-#ifdef OBJ_ELF
   if (*input_line_pointer != ',')
     {
-      as_bad ("Expected command (and alignment) after common length");
+      as_bad ("Expected comma after common length");
       ignore_rest_of_line ();
       return;
     }
   input_line_pointer++;
-  temp = get_absolute_expression ();
-  if (temp > max_alignment)
+  SKIP_WHITESPACE ();
+  if (*input_line_pointer != '"')
     {
-      temp = max_alignment;
-      as_warn ("Common alignment too large: %d. assumed", temp);
-    }
-  else if (temp < 0)
-    {
-      temp = 0;
-      as_warn ("Common alignment negative; 0 assumed");
-    }
-  record_alignment (S_GET_SEGMENT (symbolP), temp);
-#else
-  if (strncmp (input_line_pointer, ",\"bss\"", 6) != 0
-      && strncmp (input_line_pointer, ",\"data\"", 7) != 0)
-    {
-      p = input_line_pointer;
-      while (*p && *p != '\n')
-	p++;
-      c = *p;
-      *p = '\0';
-      as_bad ("bad .common segment: `%s'", input_line_pointer);
-      *p = c;
-      return;
-    }
-  input_line_pointer += 6 + (input_line_pointer[2] == 'd');	/* Skip either */
+      temp = get_absolute_expression ();
+#ifndef OBJ_ELF
+      if (temp > max_alignment)
+	{
+	  temp = max_alignment;
+	  as_warn ("Common alignment too large: %d. assumed", temp);
+	}
 #endif
+      if (temp < 0)
+	{
+	  temp = 0;
+	  as_warn ("Common alignment negative; 0 assumed");
+	}
+#ifdef OBJ_ELF
+      if (symbolP->local)
+	{
+	  segT old_sec;
+	  int old_subsec;
+	  char *p;
+	  int align;
+
+	allocate_bss:
+	  old_sec = now_seg;
+	  old_subsec = now_subseg;
+	  align = temp;
+	  record_alignment (bss_section, align);
+	  subseg_set (bss_section, 0);
+	  if (align)
+	    frag_align (align, 0);
+	  if (S_GET_SEGMENT (symbolP) == bss_section)
+	    symbolP->sy_frag->fr_symbol = 0;
+	  symbolP->sy_frag = frag_now;
+	  p = frag_var (rs_org, 1, 1, (relax_substateT) 0, symbolP, size,
+			(char *) 0);
+	  *p = 0;
+	  S_SET_SEGMENT (symbolP, bss_section);
+	  S_CLEAR_EXTERNAL (symbolP);
+	  subseg_set (old_sec, old_subsec);
+	}
+      else
+#endif
+	{
+	allocate_common:
+	  S_SET_VALUE (symbolP, (valueT) size);
+	  S_SET_EXTERNAL (symbolP);
+	  /* should be common, but this is how gas does it for now */
+	  S_SET_SEGMENT (symbolP, &bfd_und_section);
+	}
+    }
+  else
+    {
+      input_line_pointer++;
+      /* @@ Some use the dot, some don't.  Can we get some consistency??  */
+      if (*input_line_pointer == '.')
+	input_line_pointer++;
+      /* @@ Some say data, some say bss.  */
+      if (strncmp (input_line_pointer, "bss\"", 4)
+	  && strncmp (input_line_pointer, "data\"", 5))
+	{
+	  while (*--input_line_pointer != '"')
+	    ;
+	  input_line_pointer--;
+	  goto bad_common_segment;
+	}
+      while (*input_line_pointer++ != '"')
+	;
+      goto allocate_common;
+    }
   demand_empty_rest_of_line ();
   return;
-}				/* s_common() */
+
+  {
+  bad_common_segment:
+    p = input_line_pointer;
+    while (*p && *p != '\n')
+      p++;
+    c = *p;
+    *p = '\0';
+    as_bad ("bad .common segment %s", input_line_pointer + 1);
+    *p = c;
+    input_line_pointer = p;
+    ignore_rest_of_line ();
+    return;
+  }
+}
 
 static void
-s_seg ()
+s_seg (ignore)
+     int ignore;
 {
 
   if (strncmp (input_line_pointer, "\"text\"", 6) == 0)
     {
       input_line_pointer += 6;
-      s_text ();
+      s_text (0);
       return;
     }
   if (strncmp (input_line_pointer, "\"data\"", 6) == 0)
     {
       input_line_pointer += 6;
-      s_data ();
+      s_data (0);
       return;
     }
   if (strncmp (input_line_pointer, "\"data1\"", 7) == 0)
@@ -414,29 +472,25 @@ s_seg ()
     }
   as_bad ("Unknown segment type");
   demand_empty_rest_of_line ();
-  return;
-}				/* s_seg() */
+}
 
 static void
 s_data1 ()
 {
   subseg_set (data_section, 1);
   demand_empty_rest_of_line ();
-  return;
-}				/* s_data1() */
+}
 
 static void
-s_proc ()
+s_proc (ignore)
+     int ignore;
 {
-  extern char is_end_of_line[];
-
-  while (!is_end_of_line[*input_line_pointer])
+  while (!is_end_of_line[(unsigned char) *input_line_pointer])
     {
       ++input_line_pointer;
     }
   ++input_line_pointer;
-  return;
-}				/* s_proc() */
+}
 
 
 /* This function is called once, at assembler startup time.  It should
@@ -444,19 +498,17 @@ s_proc ()
 void
 md_begin ()
 {
-  register char *retval = NULL;
+  register const char *retval = NULL;
   int lose = 0;
   register unsigned int i = 0;
 
   op_hash = hash_new ();
-  if (op_hash == NULL)
-    as_fatal ("Virtual memory exhausted");
 
   while (i < NUMOPCODES)
     {
       const char *name = sparc_opcodes[i].name;
       retval = hash_insert (op_hash, name, &sparc_opcodes[i]);
-      if (retval != NULL && *retval != '\0')
+      if (retval != NULL)
 	{
 	  fprintf (stderr, "internal error: can't hash `%s': %s\n",
 		   sparc_opcodes[i].name, retval);
@@ -490,13 +542,7 @@ md_begin ()
 
 
   target_big_endian = 1;
-}				/* md_begin() */
-
-void
-md_end ()
-{
-  return;
-}				/* md_end() */
+}
 
 void
 md_assemble (str)
@@ -510,7 +556,7 @@ md_assemble (str)
 
   /* See if "set" operand is absolute and small; skip sethi if so. */
   if (special_case == SPECIAL_CASE_SET
-      && the_insn.exp.X_seg == absolute_section)
+      && the_insn.exp.X_op == O_constant)
     {
       if (the_insn.exp.X_add_number >= -(1 << 12)
 	  && the_insn.exp.X_add_number < (1 << 12))
@@ -525,19 +571,17 @@ md_assemble (str)
 
   toP = frag_more (4);
   /* put out the opcode */
-  md_number_to_chars (toP, the_insn.opcode, 4);
+  md_number_to_chars (toP, (valueT) the_insn.opcode, 4);
 
   /* put out the symbol-dependent stuff */
   if (the_insn.reloc != BFD_RELOC_NONE)
     {
-      fix_new (frag_now,	/* which frag */
-	       (toP - frag_now->fr_literal),	/* where */
-	       4,		/* size */
-	       the_insn.exp.X_add_symbol,
-	       the_insn.exp.X_subtract_symbol,
-	       the_insn.exp.X_add_number,
-	       the_insn.pcrel,
-	       the_insn.reloc);
+      fix_new_exp (frag_now,	/* which frag */
+		   (toP - frag_now->fr_literal),	/* where */
+		   4,		/* size */
+		   &the_insn.exp,
+		   the_insn.pcrel,
+		   the_insn.reloc);
     }
 
   switch (special_case)
@@ -546,22 +590,19 @@ md_assemble (str)
       special_case = 0;
       assert (the_insn.reloc == BFD_RELOC_HI22);
       /* See if "set" operand has no low-order bits; skip OR if so. */
-      if (the_insn.exp.X_seg == absolute_section
+      if (the_insn.exp.X_op == O_constant
 	  && ((the_insn.exp.X_add_number & 0x3FF) == 0))
 	return;
       toP = frag_more (4);
       rsd = (the_insn.opcode >> 25) & 0x1f;
       the_insn.opcode = 0x80102000 | (rsd << 25) | (rsd << 14);
-      md_number_to_chars (toP, the_insn.opcode, 4);
-      fix_new (frag_now,	/* which frag */
-	       (toP - frag_now->fr_literal),	/* where */
-	       4,		/* size */
-	       the_insn.exp.X_add_symbol,
-	       the_insn.exp.X_subtract_symbol,
-	       the_insn.exp.X_add_number,
-	       the_insn.pcrel,
-	       BFD_RELOC_LO10
-	       );
+      md_number_to_chars (toP, (valueT) the_insn.opcode, 4);
+      fix_new_exp (frag_now,	/* which frag */
+		   (toP - frag_now->fr_literal),	/* where */
+		   4,		/* size */
+		   &the_insn.exp,
+		   the_insn.pcrel,
+		   BFD_RELOC_LO10);
       return;
 
     case SPECIAL_CASE_FDIV:
@@ -575,7 +616,7 @@ md_assemble (str)
       toP = frag_more (4);
       rsd = (the_insn.opcode >> 25) & 0x1f;
       the_insn.opcode = 0x81A00020 | (rsd << 25) | rsd;	/* fmovs dest,dest */
-      md_number_to_chars (toP, the_insn.opcode, 4);
+      md_number_to_chars (toP, (valueT) the_insn.opcode, 4);
       return;
 
     case 0:
@@ -584,7 +625,7 @@ md_assemble (str)
     default:
       as_fatal ("failed sanity check.");
     }
-}				/* md_assemble() */
+}
 
 static void
 sparc_ip (str)
@@ -897,14 +938,9 @@ sparc_ip (str)
 		char format;
 
 		if (*s++ == '%'
-
 		    && ((format = *s) == 'f')
-
 		    && isdigit (*++s))
 		  {
-
-
-
 		    for (mask = 0; isdigit (*s); ++s)
 		      {
 			mask = 10 * mask + (*s - '0');
@@ -926,15 +962,16 @@ sparc_ip (str)
 			break;
 		      }		/* register must be multiple of 4 */
 
-		    if (format == 'f')
+		    if (mask >= 32)
 		      {
-			if (mask >= 32)
-			  {
-			    error_message = ": There are only 32 f registers; [0-31]";
-			    goto error;
-			  }	/* on error */
-		      }		/* if not an 'f' register. */
-		  }		/* on error */
+			error_message = ": There are only 32 f registers; [0-31]";
+			goto error;
+		      }	/* on error */
+		  }
+		else
+		  {
+		    break;
+		  }	/* if not an 'f' register. */
 
 		switch (*args)
 		  {
@@ -990,8 +1027,7 @@ sparc_ip (str)
 	      goto immediate;
 
 	    case 'i':		/* 13 bit immediate */
-	      /* What's the difference between base13 and 13?  */
-	      the_insn.reloc = BFD_RELOC_SPARC_BASE13;
+	      the_insn.reloc = BFD_RELOC_SPARC13;
 	      immediate_max = 0x0FFF;
 
 	      /*FALLTHROUGH */
@@ -1014,17 +1050,15 @@ sparc_ip (str)
 		  else
 		    break;
 		}
-	      /* Note that if the getExpression() fails, we
-		 will still have created U entries in the
-		 symbol table for the 'symbols' in the input
-		 string.  Try not to create U symbols for
-		 registers, etc. */
+	      /* Note that if the getExpression() fails, we will still
+		 have created U entries in the symbol table for the
+		 'symbols' in the input string.  Try not to create U
+		 symbols for registers, etc.  */
 	      {
-		/* This stuff checks to see if the
-		   expression ends in +%reg If it does,
-		   it removes the register from the
-		   expression, and re-sets 's' to point
-		   to the right place */
+		/* This stuff checks to see if the expression ends in
+		   +%reg.  If it does, it removes the register from
+		   the expression, and re-sets 's' to point to the
+		   right place.  */
 
 		char *s1;
 
@@ -1055,20 +1089,46 @@ sparc_ip (str)
 	      (void) getExpression (s);
 	      s = expr_end;
 
-	      /* Check for invalid constant values.  Don't
-		 warn if constant was inside %hi or %lo,
-		 since these truncate the constant to
-		 fit.  */
-	      if (immediate_max != 0
-		  && the_insn.reloc != BFD_RELOC_LO10
-		  && the_insn.reloc != BFD_RELOC_HI22
+	      if (the_insn.exp.X_op == O_constant
 		  && the_insn.exp.X_add_symbol == 0
-		  && the_insn.exp.X_subtract_symbol == 0
-		  && the_insn.exp.X_seg == absolute_section
-		  && (the_insn.exp.X_add_number > immediate_max
-		      || the_insn.exp.X_add_number < ~immediate_max))
-		as_bad ("constant value must be between %ld and %ld",
-			~immediate_max, immediate_max);
+		  && the_insn.exp.X_op_symbol == 0)
+		{
+		  /* For pc-relative call instructions, we reject
+		     constants to get better code.  */
+		  if (the_insn.pcrel
+		      && the_insn.reloc == BFD_RELOC_32_PCREL_S2
+		      && in_signed_range (the_insn.exp.X_add_number, 0x3fff)
+		      )
+		    {
+		      error_message = ": PC-relative operand can't be a constant";
+		      goto error;
+		    }
+		  /* Check for invalid constant values.  Don't warn if
+		     constant was inside %hi or %lo, since these
+		     truncate the constant to fit.  */
+		  if (immediate_max != 0
+		      && the_insn.reloc != BFD_RELOC_LO10
+		      && the_insn.reloc != BFD_RELOC_HI22
+		      && !in_signed_range (the_insn.exp.X_add_number,
+					   immediate_max)
+		      )
+		    {
+		      if (the_insn.pcrel)
+			/* Who knows?  After relocation, we may be within
+			   range.  Let the linker figure it out.  */
+			{
+			  the_insn.exp.X_op = O_symbol;
+			  the_insn.exp.X_add_symbol = section_symbol (absolute_section);
+			}
+		      else
+			/* Immediate value is non-pcrel, and out of
+                           range.  */
+			as_bad ("constant value %ld out of range (%ld .. %ld)",
+				the_insn.exp.X_add_number,
+				~immediate_max, immediate_max);
+		    }
+		}
+
 	      /* Reset to prevent extraneous range check.  */
 	      immediate_max = 0;
 
@@ -1089,7 +1149,8 @@ sparc_ip (str)
 
 		input_line_pointer = s;
 
-		if (expression (&e) == absolute_section)
+		expression (&e);
+		if (e.X_op == O_constant)
 		  {
 		    opcode |= e.X_add_number << 5;
 		    s = input_line_pointer;
@@ -1212,8 +1273,7 @@ sparc_ip (str)
     }				/* forever looking for a match */
 
   the_insn.opcode = opcode;
-  return;
-}				/* sparc_ip() */
+}
 
 static int
 getExpression (str)
@@ -1225,16 +1285,11 @@ getExpression (str)
   save_in = input_line_pointer;
   input_line_pointer = str;
   seg = expression (&the_insn.exp);
-  if (seg == absolute_section
-      || seg == text_section
-      || seg == data_section
-      || seg == bss_section
-      || seg == undefined_section
-      || seg == diff_section
-      || seg == big_section
-      || seg == absent_section)
-    /* ok */;
-  else
+  if (seg != absolute_section
+      && seg != text_section
+      && seg != data_section
+      && seg != bss_section
+      && seg != undefined_section)
     {
       the_insn.error = "bad segment";
       expr_end = input_line_pointer;
@@ -1308,7 +1363,7 @@ md_atof (type, litP, sizeP)
   *sizeP = prec * sizeof (LITTLENUM_TYPE);
   for (wordP = words; prec--;)
     {
-      md_number_to_chars (litP, (long) (*wordP++), sizeof (LITTLENUM_TYPE));
+      md_number_to_chars (litP, (valueT) (*wordP++), sizeof (LITTLENUM_TYPE));
       litP += sizeof (LITTLENUM_TYPE);
     }
   return 0;
@@ -1320,79 +1375,55 @@ md_atof (type, litP, sizeP)
 void
 md_number_to_chars (buf, val, n)
      char *buf;
-     long val;
+     valueT val;
      int n;
 {
-
-  switch (n)
-    {
-
-    case 4:
-      *buf++ = val >> 24;
-      *buf++ = val >> 16;
-    case 2:
-      *buf++ = val >> 8;
-    case 1:
-      *buf = val;
-      break;
-
-    default:
-      as_fatal ("failed sanity check.");
-    }
-  return;
-}				/* md_number_to_chars() */
+  number_to_chars_bigendian (buf, val, n);
+}
 
 /* Apply a fixS to the frags, now that we know the value it ought to
    hold. */
 
-#ifdef BFD_ASSEMBLER
 int
-#else
-void
-#endif
 md_apply_fix (fixP, value)
      fixS *fixP;
-#ifdef BFD_ASSEMBLER
-     long *value;
-#else
-     long value;
-#endif
+     valueT *value;
 {
   char *buf = fixP->fx_where + fixP->fx_frag->fr_literal;
-  long val;
+  offsetT val;
 
-#ifdef BFD_ASSEMBLER
   val = *value;
-#else
-  val = value;
-#endif
 
-  assert (fixP->fx_size == 4);
-#ifdef BFD_ASSEMBLER
   assert (fixP->fx_r_type < BFD_RELOC_UNUSED);
-#else
-  assert (fixP->fx_r_type < NO_RELOC);
-#endif
 
   fixP->fx_addnumber = val;	/* Remember value for emit_reloc */
 
-  /*
-   * This is a hack.  There should be a better way to
-   * handle this.
-   */
+#ifdef OBJ_ELF
+  /* FIXME: SPARC ELF relocations don't use an addend in the data
+     field itself.  This whole approach should be somehow combined
+     with the calls to bfd_perform_relocation.  */
+  if (fixP->fx_addsy != NULL)
+    return 1;
+#endif
+
+  /* This is a hack.  There should be a better way to
+     handle this.  Probably in terms of howto fields, once
+     we can look at these fixups in terms of howtos.  */
   if (fixP->fx_r_type == BFD_RELOC_32_PCREL_S2 && fixP->fx_addsy)
-    {
-      val += fixP->fx_where + fixP->fx_frag->fr_address;
-    }
+    val += fixP->fx_where + fixP->fx_frag->fr_address;
 
   switch (fixP->fx_r_type)
     {
+    case BFD_RELOC_16:
+      buf[0] = val >> 8;
+      buf[1] = val;
+      break;
 
     case BFD_RELOC_32:
-      buf[0] = 0;		/* val >> 24; */
-      buf[1] = 0;		/* val >> 16; */
-      buf[2] = 0;		/* val >> 8; */
-      buf[3] = 0;		/* val; */
+      buf[0] = val >> 24;
+      buf[1] = val >> 16;
+      buf[2] = val >> 8;
+      buf[3] = val;
       break;
 
     case BFD_RELOC_32_PCREL_S2:
@@ -1428,15 +1459,6 @@ md_apply_fix (fixP, value)
       buf[3] = val & 0xff;
       break;
 
-    case BFD_RELOC_SPARC13:
-      if (val & ~0x00001fff)
-	{
-	  as_bad ("relocation overflow");
-	}			/* on overflow */
-      buf[2] |= (val >> 8) & 0x1f;
-      buf[3] = val & 0xff;
-      break;
-
 
     case BFD_RELOC_LO10:
       if (!fixP->fx_addsy)
@@ -1447,12 +1469,11 @@ md_apply_fix (fixP, value)
       else
 	buf[3] = 0;
       break;
-    case BFD_RELOC_SPARC_BASE13:
-      if (((val > 0) && (val & ~0x00001fff))
-	  || ((val < 0) && (~(val - 1) & ~0x00001fff)))
-	{
-	  as_bad ("relocation overflow");
-	}			/* on overflow */
+
+    case BFD_RELOC_SPARC13:
+      if (! in_signed_range (val, 0x1fff))
+	as_bad ("relocation overflow");
+
       buf[2] |= (val >> 8) & 0x1f;
       buf[3] = val;
       break;
@@ -1472,25 +1493,13 @@ md_apply_fix (fixP, value)
       break;
     }
 
-#ifdef BFD_ASSEMBLER
+  /* Are we finished with this relocation now?  */
+  if (fixP->fx_addsy == 0 && !fixP->fx_pcrel)
+    fixP->fx_done = 1;
+
   return 1;
-#endif
 }
 
-/* should never be called for sparc */
-void
-md_create_short_jump (ptr, from_addr, to_addr, frag, to_symbol)
-     char *ptr;
-     long from_addr;
-     long to_addr;
-     fragS *frag;
-     symbolS *to_symbol;
-{
-  as_fatal ("sparc_create_short_jmp\n");
-}
-
-#ifdef BFD_ASSEMBLER
-
 /* Translate internal representation of relocation info to BFD target
    format.  */
 arelent *
@@ -1506,125 +1515,47 @@ tc_gen_reloc (section, fixp)
 
   reloc->sym_ptr_ptr = &fixp->fx_addsy->bsym;
   reloc->address = fixp->fx_frag->fr_address + fixp->fx_where;
-  if (fixp->fx_pcrel == 0)
-    reloc->addend = fixp->fx_addnumber;
-  else
-    switch (OUTPUT_FLAVOR)
-      {
-      case bfd_target_elf_flavour:
-	reloc->addend = 0;
-	break;
-      case bfd_target_aout_flavour:
-	reloc->addend = - reloc->address;
-	break;
-      default:
-	/* What's a good default here?  Is there any??  */
-	abort ();
-      }
 
   switch (fixp->fx_r_type)
     {
+    case BFD_RELOC_16:
     case BFD_RELOC_32:
     case BFD_RELOC_HI22:
     case BFD_RELOC_LO10:
     case BFD_RELOC_32_PCREL_S2:
+    case BFD_RELOC_SPARC13:
     case BFD_RELOC_SPARC_BASE13:
+    case BFD_RELOC_SPARC_WDISP22:
       code = fixp->fx_r_type;
       break;
     default:
       abort ();
     }
   reloc->howto = bfd_reloc_type_lookup (stdoutput, code);
-  assert (reloc->howto != 0);
+  if (reloc->howto == 0)
+    {
+      as_bad_where (fixp->fx_file, fixp->fx_line,
+		    "internal error: can't export reloc type %d",
+		    fixp->fx_r_type);
+      return 0;
+    }
+  assert (!fixp->fx_pcrel == !reloc->howto->pc_relative);
+
+  /* @@ Why fx_addnumber sometimes and fx_offset other times?  */
+  if (reloc->howto->pc_relative == 0)
+    reloc->addend = fixp->fx_addnumber;
+#if defined (OBJ_ELF) || defined (OBJ_COFF)
+  else if ((fixp->fx_addsy->bsym->flags & BSF_SECTION_SYM) != 0)
+    reloc->addend = (section->vma
+		     + fixp->fx_addnumber
+		     + md_pcrel_from (fixp));
+#endif
+  else
+    reloc->addend = fixp->fx_offset - reloc->address;
 
   return reloc;
 }
 
-#else
-
-/* Translate internal representation of relocation info to target format.
-
-   On sparc: first 4 bytes are normal unsigned long address, next three
-   bytes are index, most sig. byte first.  Byte 7 is broken up with
-   bit 7 as external, bits 6 & 5 unused, and the lower
-   five bits as relocation type.  Next 4 bytes are long addend. */
-/* Thanx and a tip of the hat to Michael Bloom, mb@ttidca.tti.com */
-void
-tc_aout_fix_to_chars (where, fixP, segment_address_in_file)
-     char *where;
-     fixS *fixP;
-     relax_addressT segment_address_in_file;
-{
-  long r_index;
-  long r_extern;
-  long r_addend = 0;
-  long r_address;
-
-  know (fixP->fx_addsy);
-
-  if (!S_IS_DEFINED (fixP->fx_addsy))
-    {
-      r_extern = 1;
-      r_index = fixP->fx_addsy->sy_number;
-    }
-  else
-    {
-      r_extern = 0;
-      r_index = S_GET_TYPE (fixP->fx_addsy);
-    }
-
-  /* this is easy */
-  md_number_to_chars (where,
-		      r_address = fixP->fx_frag->fr_address + fixP->fx_where - segment_address_in_file,
-		      4);
-
-  /* now the fun stuff */
-  where[4] = (r_index >> 16) & 0x0ff;
-  where[5] = (r_index >> 8) & 0x0ff;
-  where[6] = r_index & 0x0ff;
-  where[7] = ((r_extern << 7) & 0x80) | (0 & 0x60) | (fixP->fx_r_type & 0x1F);
-
-  /* Also easy */
-  if (fixP->fx_addsy->sy_frag)
-    {
-      r_addend = fixP->fx_addsy->sy_frag->fr_address;
-    }
-
-  if (fixP->fx_pcrel)
-    {
-      r_addend += fixP->fx_offset - r_address;
-    }
-  else
-    {
-      r_addend = fixP->fx_addnumber;
-    }
-
-  md_number_to_chars (&where[8], r_addend, 4);
-
-  return;
-}				/* tc_aout_fix_to_chars() */
-#endif
-
-/* should never be called for sparc */
-void
-md_create_long_jump (ptr, from_addr, to_addr, frag, to_symbol)
-     char *ptr;
-     long from_addr, to_addr;
-     fragS *frag;
-     symbolS *to_symbol;
-{
-  as_fatal ("sparc_create_long_jump\n");
-}				/* md_create_long_jump() */
-
-/* should never be called for sparc */
-int
-md_estimate_size_before_relax (fragP, segtype)
-     fragS *fragP;
-     segT segtype;
-{
-  as_fatal ("sparc_estimate_size_before_relax\n");
-  return (1);
-}				/* md_estimate_size_before_relax() */
 
 #if 0
 /* for debugging only */
@@ -1632,8 +1563,7 @@ static void
 print_insn (insn)
      struct sparc_it *insn;
 {
-  char *Reloc[] =
-  {
+  const char *const Reloc[] = {
     "RELOC_8",
     "RELOC_16",
     "RELOC_32",
@@ -1662,9 +1592,7 @@ print_insn (insn)
   };
 
   if (insn->error)
-    {
-      fprintf (stderr, "ERROR: %s\n");
-    }
+    fprintf (stderr, "ERROR: %s\n");
   fprintf (stderr, "opcode=0x%08x\n", insn->opcode);
   fprintf (stderr, "reloc = %s\n", Reloc[insn->reloc]);
   fprintf (stderr, "exp = {\n");
@@ -1675,17 +1603,15 @@ print_insn (insn)
 	       : "???")
 	    : "0"));
   fprintf (stderr, "\t\tX_sub_symbol = %s\n",
-	   ((insn->exp.X_subtract_symbol != NULL)
-	    ? (S_GET_NAME (insn->exp.X_subtract_symbol)
-	       ? S_GET_NAME (insn->exp.X_subtract_symbol)
+	   ((insn->exp.X_op_symbol != NULL)
+	    ? (S_GET_NAME (insn->exp.X_op_symbol)
+	       ? S_GET_NAME (insn->exp.X_op_symbol)
 	       : "???")
 	    : "0"));
   fprintf (stderr, "\t\tX_add_number = %d\n",
 	   insn->exp.X_add_number);
   fprintf (stderr, "}\n");
-  return;
-}				/* print_insn() */
-
+}
 #endif
 
 /*
@@ -1728,7 +1654,6 @@ md_parse_option (argP, cntP, vecP)
   if (!strcmp (*argP, "bump"))
     {
       warn_on_bump = 1;
-
     }
   else if (**argP == 'A')
     {
@@ -1752,6 +1677,29 @@ md_parse_option (argP, cntP, vecP)
 	  architecture_requested = 1;
 	}
     }
+#ifdef OBJ_ELF
+  else if (**argP == 'V')
+    {
+      print_version_id ();
+    }
+  else if (**argP == 'Q')
+    {
+      /* Qy - do emit .comment
+	 Qn - do not emit .comment */
+    }
+  else if (**argP == 's')
+    {
+      /* use .stab instead of .stab.excl */
+    }
+  else if (**argP == 'q')
+    {
+      /* quick -- native assembler does fewer checks */
+    }
+#endif
+  else if (strcmp (*argP, "sparc") == 0)
+    {
+      /* Ignore -sparc, used by SunOS make default .s.o rule.  */
+    }
   else
     {
       /* Unknown option */
@@ -1784,13 +1732,23 @@ md_operand (expressionP)
 }
 
 /* Round up a section size to the appropriate boundary. */
-long 
+valueT
 md_section_align (segment, size)
      segT segment;
-     long size;
+     valueT size;
 {
-  /* Round all sects to multiple of 8 */
-  return (size + 7) & ~7;
+#ifndef OBJ_ELF
+  /* This is not right for ELF; a.out wants it, and COFF will force
+     the alignment anyways.  */
+  valueT align = (valueT) 1 << (valueT) (stdoutput->xvec->align_power_min);
+  valueT newsize;
+  /* turn alignment value into a mask */
+  align--;
+  newsize = (size + align) & ~align;
+  return newsize;
+#else
+  return size;
+#endif
 }
 
 /* Exactly what point is a PC-relative offset relative TO?
@@ -1804,13 +1762,4 @@ md_pcrel_from (fixP)
   return fixP->fx_size + fixP->fx_where + fixP->fx_frag->fr_address;
 }
 
-#ifndef BFD_ASSEMBLER
-void 
-tc_aout_pre_write_hook (headers)
-     object_headers *headers;
-{
-  H_SET_VERSION (headers, 1);
-}
-#endif
-
 /* end of tc-sparc.c */
