@@ -358,32 +358,18 @@ md_begin ()
   char *errorval = 0;
   int synthetic_too = 1;	/* If 0, just use real opcodes. */
 
-  if ((op_hash = hash_new ()))
-    {
-      for (tP = totstrs; *tP->name && !*errorval; tP++)
-	{
-	  errorval = hash_insert (op_hash, tP->name, &tP->detail);
-	}
-      if (synthetic_too)
-	{
-	  for (tP = synthetic_totstrs; *tP->name && !*errorval; tP++)
-	    {
-	      errorval = hash_insert (op_hash, tP->name, &tP->detail);
-	    }
-	}
-    }
-  else
-    {
-      errorval = "Virtual memory exceeded";
-    }
+  op_hash = hash_new ();
+
+  for (tP = totstrs; *tP->name && !errorval; tP++)
+    errorval = hash_insert (op_hash, tP->name, &tP->detail);
+
+  if (synthetic_too)
+    for (tP = synthetic_totstrs; *tP->name && !errorval; tP++)
+      errorval = hash_insert (op_hash, tP->name, &tP->detail);
+
   if (errorval)
     as_fatal (errorval);
 }
-
-void
-md_end ()
-{
-}				/* md_end */
 
 int
 md_parse_option (argP, cntP, vecP)
@@ -460,18 +446,7 @@ md_number_to_chars (con, value, nbytes)
      valueT value;		/* The value of the bits. */
      int nbytes;		/* Number of bytes in the output. */
 {
-  int n = nbytes;
-  valueT v = value;
-
-  con += nbytes - 1;		/* Tahoes is (Bleah!) big endian */
-  while (nbytes--)
-    {
-      *con-- = value;		/* Lint wants & MASK_CHAR. */
-      value >>= BITS_PER_CHAR;
-    }
-  /* XXX line number probably botched for this warning message. */
-  if (value != 0 && value != -1)
-    as_warn ("Displacement (%ld) long for instruction field length (%d).", v, n);
+  number_to_chars_bigendian (con, value, nbytes);
 }
 
 #ifdef comment
@@ -491,11 +466,9 @@ tc_apply_fix (fixP, val)
      fixS *fixP;
      long val;
 {
-  /*	char *place = fixP->fx_where + fixP->fx_frag->fr_literal; */
   /* should never be called */
   know (0);
-  return;
-}				/* tc_apply_fix() */
+}
 
 void				/* Knows about order of bytes in address. */
 md_number_to_disp (con, value, nbytes)
@@ -585,9 +558,7 @@ tc_aout_fix_to_chars (where, fixP, segment_address_in_file)
 		       ? 2
 		       : 42)))) << 5) & 0x60)
 	      | ((!S_IS_DEFINED (fixP->fx_addsy) << 4) & 0x10));
-
-  return;
-}				/* tc_aout_fix_to_chars() */
+}
 
 /* Relocate byte stuff */
 
