@@ -1,4 +1,4 @@
-/* tc-z8k.c -- Assemble code for the Zilog Z800N
+/* tc-z8k.c -- Assemble code for the Zilog Z800n
    Copyright (C) 1992 Free Software Foundation.
 
    This file is part of GAS, the GNU Assembler.
@@ -21,9 +21,9 @@
   Written By Steve Chamberlain
   sac@cygnus.com
   */
-
-#include <stdio.h>
 #define DEFINE_TABLE
+#include <stdio.h>
+
 #include "../opcodes/z8k-opc.h"
 
 #include "as.h"
@@ -34,12 +34,13 @@ const char comment_chars[] =
 {'!', 0};
 const char line_separator_chars[] =
 {';', 0};
-const char line_comment_chars[] = { '#', 0};
+const char line_comment_chars[] =
+{'#', 0};
 
 extern int machine;
 extern int coff_flags;
 int segmented_mode;
-int md_reloc_size;
+const int md_reloc_size;
 
 /* This table describes all the machine specific pseudo-ops the assembler
    has to support.  The fields are:
@@ -66,70 +67,81 @@ s_unseg ()
   coff_flags = F_Z8002;
 }
 
-static 
-void even()
+static
+void
+even ()
 {
   frag_align (1, 0);
-  record_alignment(now_seg,1);
+  record_alignment (now_seg, 1);
 }
-void obj_coff_section();
 
-int tohex(c)
-int c;
+void obj_coff_section ();
+
+int
+tohex (c)
+     int c;
 {
-if (isdigit(c)) return c - '0';
-if (islower(c)) return c - 'a' + 10;
-return c - 'A' + 10;
+  if (isdigit (c))
+    return c - '0';
+  if (islower (c))
+    return c - 'a' + 10;
+  return c - 'A' + 10;
 }
-void sval()
+
+void
+sval ()
 {
 
-  SKIP_WHITESPACE();
-  if (*input_line_pointer == '\'') {
-    int c;
-    input_line_pointer++;
-    c = *input_line_pointer++;
-    while (c != '\'') {
-      if (c== '%') {
-	c = (tohex(input_line_pointer[0])  << 4)
-	 | tohex(input_line_pointer[1]);
-	input_line_pointer+=2;
-      }
-      FRAG_APPEND_1_CHAR(c);
+  SKIP_WHITESPACE ();
+  if (*input_line_pointer == '\'')
+    {
+      int c;
+      input_line_pointer++;
       c = *input_line_pointer++;
+      while (c != '\'')
+	{
+	  if (c == '%')
+	    {
+	      c = (tohex (input_line_pointer[0]) << 4)
+		| tohex (input_line_pointer[1]);
+	      input_line_pointer += 2;
+	    }
+	  FRAG_APPEND_1_CHAR (c);
+	  c = *input_line_pointer++;
+	}
+      demand_empty_rest_of_line ();
     }
-    demand_empty_rest_of_line();
-  }
 
 }
 const pseudo_typeS md_pseudo_table[] =
 {
- {"int", cons, 2},
- {"data.b", cons, 1},
- {"data.w", cons, 2},
- {"data.l", cons, 4},
- {"form", listing_psize, 0},
- {"heading", listing_title, 0},
- {"import", s_ignore, 0},
- {"page", listing_eject, 0},
- {"program", s_ignore, 0},
- {"z8001", s_segm, 0},
- {"z8002", s_unseg, 0},
+  {"int", cons, 2},
+  {"data.b", cons, 1},
+  {"data.w", cons, 2},
+  {"data.l", cons, 4},
+  {"form", listing_psize, 0},
+  {"heading", listing_title, 0},
+  {"import", s_ignore, 0},
+  {"page", listing_eject, 0},
+  {"program", s_ignore, 0},
+  {"z8001", s_segm, 0},
+  {"z8002", s_unseg, 0},
 
 
- {"segm", s_segm, 0},
- {"unsegm", s_unseg, 0},
- {"name", s_app_file, 0},
- {"global",s_globl,0},
- {"wval",cons,2},
- {"lval",cons,4},
- {"bval",cons,1},
- {"sval",sval,0},
- {"rsect",obj_coff_section,0},
- {"sect",obj_coff_section,0},
- {"block",s_space,0},
- {"even",even,0},
- {0, 0, 0}
+  {"segm", s_segm, 0},
+  {"unsegm", s_unseg, 0},
+  {"unseg", s_unseg, 0},
+  {"name", s_app_file, 0},
+  {"global", s_globl, 0},
+  {"wval", cons, 2},
+  {"lval", cons, 4},
+  {"bval", cons, 1},
+  {"sval", sval, 0},
+  {"rsect", obj_coff_section, 0},
+  {"sect", obj_coff_section, 0},
+  {"block", s_space, 0},
+  {"even", even, 0},
+  {0, 0, 0}
 };
 
 const char EXP_CHARS[] = "eE";
@@ -170,17 +182,15 @@ md_begin ()
   s_unseg ();
 
   /* insert the pseudo ops too */
-  for (idx = 0; md_pseudo_table[idx].poc_name; idx++) 
-  {
-    opcode_entry_type *fake_opcode;
-    fake_opcode = (opcode_entry_type*)malloc(sizeof(opcode_entry_type));
-    fake_opcode->name = md_pseudo_table[idx].poc_name,
-    fake_opcode->func = (void *)(md_pseudo_table+idx);
-    fake_opcode->opcode = 250;
-
-    hash_insert(opcode_hash_control,fake_opcode->name,fake_opcode);
-    
-  }
+  for (idx = 0; md_pseudo_table[idx].poc_name; idx++)
+    {
+      opcode_entry_type *fake_opcode;
+      fake_opcode = (opcode_entry_type *) malloc (sizeof (opcode_entry_type));
+      fake_opcode->name = md_pseudo_table[idx].poc_name,
+	fake_opcode->func = (void *) (md_pseudo_table + idx);
+      fake_opcode->opcode = 250;
+      hash_insert (opcode_hash_control, fake_opcode->name, fake_opcode);
+    }
 }
 
 struct z8k_exp
@@ -207,6 +217,9 @@ static expressionS *imm_operand;
 
 int reg[16];
 int the_cc;
+int the_ctrl;
+int the_flags;
+int the_interrupt;
 
 char *
 DEFUN (whatreg, (reg, src),
@@ -249,48 +262,66 @@ DEFUN (parse_reg, (src, mode, reg),
        unsigned int *reg)
 {
   char *res = 0;
+  char regno;
 
-  if (src[0] == 's' && src[1]=='p') {
-    if (segmented_mode) {
-      *mode = CLASS_REG_LONG;
-      *reg = 14;
-    }
-    else 
+  if (src[0] == 's' && src[1] == 'p')
     {
-      *mode = CLASS_REG_WORD;
-      *reg = 15;
+      if (segmented_mode)
+        {
+          *mode = CLASS_REG_LONG;
+          *reg = 14;
+        }
+      else
+        {
+          *mode = CLASS_REG_WORD;
+          *reg = 15;
+        }
+      return src + 2;
     }
-    return src+2;
-  }
   if (src[0] == 'r')
-  {
-    if (src[1] == 'r')
     {
-      *mode = CLASS_REG_LONG;
-      res = whatreg (reg, src + 2);
+      if (src[1] == 'r')
+        {
+          *mode = CLASS_REG_LONG;
+          res = whatreg (reg, src + 2);
+	  regno = *reg;
+	  if (regno > 14)
+          	as_warn ("register rr%d, out of range.",regno);
+        }
+      else if (src[1] == 'h')
+        {
+          *mode = CLASS_REG_BYTE;
+          res = whatreg (reg, src + 2);
+	  regno = *reg;
+	  if (regno > 7)
+          	as_warn ("register rh%d, out of range.",regno);
+        }
+      else if (src[1] == 'l')
+        {
+          *mode = CLASS_REG_BYTE;
+          res = whatreg (reg, src + 2);
+	  regno = *reg;
+	  if (regno > 7)
+          	as_warn ("register rl%d, out of range.",regno);
+          *reg += 8;
+        }
+      else if (src[1] == 'q')
+        {
+          *mode = CLASS_REG_QUAD;
+          res = whatreg (reg, src + 2);
+	  regno = *reg;
+	  if (regno > 12)
+          	as_warn ("register rq%d, out of range.",regno);
+        }
+      else
+        {
+          *mode = CLASS_REG_WORD;
+          res = whatreg (reg, src + 1);
+	  regno = *reg;
+	  if (regno > 15)
+          	as_warn ("register r%d, out of range.",regno);
+        }
     }
-    else if (src[1] == 'h' )
-    {
-      *mode = CLASS_REG_BYTE;
-      res = whatreg (reg, src + 2) ;
-    }
-    else if (src[1] == 'l'  )
-    {
-      *mode = CLASS_REG_BYTE;
-      res = whatreg (reg, src + 2);
-      *reg += 8;
-    }
-    else if (src[1] == 'q')
-    {
-      *mode = CLASS_REG_QUAD;
-      res = whatreg (reg, src + 2);
-    }
-    else
-    {
-      *mode = CLASS_REG_WORD;
-      res = whatreg (reg, src + 1);
-    }
-  }
   return res;
 
 }
@@ -302,29 +333,14 @@ DEFUN (parse_exp, (s, op),
 {
   char *save = input_line_pointer;
   char *new;
-  segT seg;
 
   input_line_pointer = s;
-  seg = expr (0, op);
+  expression (op);
+  if (op->X_op == O_absent)
+    as_bad ("missing operand");
   new = input_line_pointer;
   input_line_pointer = save;
-  if (SEG_NORMAL (seg))
-    return new;
-  switch (seg)
-    {
-    case SEG_ABSOLUTE:
-    case SEG_UNKNOWN:
-    case SEG_DIFFERENCE:
-    case SEG_BIG:
-    case SEG_REGISTER:
-      return new;
-    case SEG_ABSENT:
-      as_bad ("Missing operand");
-      return new;
-    default:
-      as_bad ("Don't understand operand of type %s", segment_name (seg));
-      return new;
-    }
+  return new;
 }
 
 /* The many forms of operand:
@@ -389,6 +405,161 @@ DEFUN (regaddr, (mode, string),
     }
 }
 
+struct ctrl_names
+{
+   int value;
+   char *name;
+};
+
+struct ctrl_names ctrl_table[] =
+{
+   0x2, "fcw",
+   0X3, "refresh",
+   0x4, "psapseg",
+   0x5, "psapoff",
+   0x5, "psap",
+   0x6, "nspseg",
+   0x7, "nspoff",
+   0x7, "nsp",
+   0, 0
+};
+   
+static void
+DEFUN (get_ctrl_operand, (ptr, mode, dst),
+       char **ptr AND
+       struct z8k_op *mode AND
+       unsigned int dst)
+{
+  char *src = *ptr;
+  int r;
+  int i;
+
+  while (*src == ' ')
+    src++;
+
+  mode->mode = CLASS_CTRL;
+  for (i = 0; ctrl_table[i].name; i++)
+    {
+      int j;
+
+      for (j = 0; ctrl_table[i].name[j]; j++)
+        {
+          if (ctrl_table[i].name[j] != src[j])
+            goto fail;
+        }
+      the_ctrl = ctrl_table[i].value;
+      *ptr = src + j;
+      return;
+    fail:;
+    }
+  the_ctrl = 0;
+  return;
+}
+
+struct flag_names
+{
+  int value;
+  char *name;
+
+};
+
+struct flag_names flag_table[] =
+{
+  0x1, "p",
+  0x1, "v",
+  0x2, "s",
+  0x4, "z",
+  0x8, "c",
+  0x0, "+",
+  0, 0
+};
+
+static void
+DEFUN (get_flags_operand, (ptr, mode, dst),
+       char **ptr AND
+       struct z8k_op *mode AND
+       unsigned int dst)
+{
+  char *src = *ptr;
+  int r;
+  int i;
+  int j;
+
+  while (*src == ' ')
+    src++;
+
+  mode->mode = CLASS_FLAGS;
+  the_flags = 0;
+  for (j = 0; j <= 9; j++)
+    {
+     if (!src[j])
+	goto done;
+     for (i = 0; flag_table[i].name; i++)
+        {
+          if (flag_table[i].name[0] == src[j])
+		{
+	        the_flags = the_flags | flag_table[i].value;
+		goto match;
+		}
+        }
+      goto done;
+    match:
+     ;
+    }
+  done:
+  *ptr = src + j;
+  return;
+}
+
+
+struct interrupt_names
+{
+  int value;
+  char *name;
+
+};
+
+struct interrupt_names intr_table[] =
+{
+  0x1, "nvi",
+  0x2, "vi",
+  0x3, "both",
+  0x3, "all",
+  0, 0
+};
+
+static void
+DEFUN (get_interrupt_operand, (ptr, mode, dst),
+       char **ptr AND
+       struct z8k_op *mode AND
+       unsigned int dst)
+{
+  char *src = *ptr;
+  int r;
+  int i;
+
+  while (*src == ' ')
+    src++;
+
+  mode->mode = CLASS_IMM;
+  for (i = 0; intr_table[i].name; i++)
+    {
+      int j;
+
+      for (j = 0; intr_table[i].name[j]; j++)
+        {
+          if (intr_table[i].name[j] != src[j])
+            goto fail;
+        }
+      the_interrupt = intr_table[i].value;
+      *ptr = src + j;
+      return;
+    fail:;
+    }
+  the_interrupt = 0x0;
+  return;
+}
+
 struct cc_names
 {
   int value;
@@ -452,14 +623,13 @@ DEFUN (get_cc_operand, (ptr, mode, dst),
     fail:;
     }
   the_cc = 0x8;
-  return;
 }
 
 static void
-DEFUN (get_operand, (ptr, mode, dst),
-       char **ptr AND
-       struct z8k_op *mode AND
-       unsigned int dst)
+get_operand (ptr, mode, dst)
+     char **ptr;
+     struct z8k_op *mode;
+     unsigned int dst;
 {
   char *src = *ptr;
   char *end;
@@ -514,7 +684,7 @@ DEFUN (get_operand, (ptr, mode, dst),
 		    }
 
 		  regaddr (mode->mode, "ra(rb) ra");
-		  regword (mode->mode, "ra(rb) rb");
+/*		  regword (mode->mode, "ra(rb) rb");*/
 		  mode->mode = CLASS_BX;
 		  mode->reg = regn;
 		  mode->x_reg = nr;
@@ -569,13 +739,13 @@ DEFUN (get_operand, (ptr, mode, dst),
 
 static
 char *
-DEFUN (get_operands, (opcode, op_end, operand),
-       opcode_entry_type * opcode AND
-       char *op_end AND
-       op_type * operand)
+get_operands (opcode, op_end, operand)
+     opcode_entry_type *opcode;
+     char *op_end;
+     op_type *operand;
 {
   char *ptr = op_end;
-
+char *savptr;
   switch (opcode->noperands)
     {
     case 0:
@@ -586,32 +756,54 @@ DEFUN (get_operands, (opcode, op_end, operand),
     case 1:
       ptr++;
       if (opcode->arg_info[0] == CLASS_CC)
-	{
-	  get_cc_operand (&ptr, operand + 0, 0);
-	}
+        {
+          get_cc_operand (&ptr, operand + 0, 0);
+        }
+      else if (opcode->arg_info[0] == CLASS_FLAGS)
+        {
+          get_flags_operand (&ptr, operand + 0, 0);
+        }
+      else if (opcode->arg_info[0] == (CLASS_IMM +(ARG_IMM2)))
+        {
+          get_interrupt_operand (&ptr, operand + 0, 0);
+        }
       else
-	{
-
-	  get_operand (&ptr, operand + 0, 0);
-	}
+        {
+          get_operand (&ptr, operand + 0, 0);
+        }
       operand[1].mode = 0;
       break;
 
     case 2:
       ptr++;
+      savptr = ptr;
       if (opcode->arg_info[0] == CLASS_CC)
-	{
-	  get_cc_operand (&ptr, operand + 0, 0);
-	}
+        {
+          get_cc_operand (&ptr, operand + 0, 0);
+        }
+      else if (opcode->arg_info[0] == CLASS_CTRL)
+             {
+               get_ctrl_operand (&ptr, operand + 0, 0);
+               if (the_ctrl == 0)
+                 {
+                   ptr = savptr;
+                   get_operand (&ptr, operand + 0, 0);
+                   if (ptr == 0)
+                     return;
+                   if (*ptr == ',')
+                     ptr++;
+                   get_ctrl_operand (&ptr, operand + 1, 1);
+                   return ptr;
+                 }
+             }
       else
-	{
-
-	  get_operand (&ptr, operand + 0, 0);
-	}
-      if(ptr == 0) 
-       return;
+        {
+          get_operand (&ptr, operand + 0, 0);
+        }
+      if (ptr == 0)
+        return;
       if (*ptr == ',')
-	ptr++;
+        ptr++;
       get_operand (&ptr, operand + 1, 1);
       break;
 
@@ -690,6 +882,19 @@ DEFUN (get_specific, (opcode, operands),
 	     something that's ok */
 		  goto fail;
 		}
+	      else if (this_try->arg_info[i] & CLASS_PR)
+		{
+		  if (mode == CLASS_REG_LONG && segmented_mode)
+		    {
+		      /* ok */
+		    }
+		  else if (mode == CLASS_REG_WORD && !segmented_mode)
+		    {
+		      /* ok */
+		    }
+		  else
+		    goto fail;
+		}
 	      else
 		goto fail;
 	    }
@@ -729,7 +934,7 @@ DEFUN (check_operand, (operand, width, string),
        char *string)
 {
   if (operand->exp.X_add_symbol == 0
-      && operand->exp.X_subtract_symbol == 0)
+      && operand->exp.X_op_symbol == 0)
     {
 
       /* No symbol involved, let's look at offset, it's dangerous if any of
@@ -753,17 +958,15 @@ DEFUN (newfix, (ptr, type, operand),
        expressionS * operand)
 {
   if (operand->X_add_symbol
-      || operand->X_subtract_symbol
+      || operand->X_op_symbol
       || operand->X_add_number)
     {
-      fix_new (frag_now,
-	       ptr,
-	       1,
-	       operand->X_add_symbol,
-	       operand->X_subtract_symbol,
-	       operand->X_add_number,
-	       0,
-	       type);
+      fix_new_exp (frag_now,
+		   ptr,
+		   1,
+		   operand,
+		   0,
+		   type);
     }
 }
 
@@ -804,9 +1007,9 @@ DEFUN (apply_fix, (ptr, type, operand, size),
  */
 #define INSERT(x,y) *x++ = y>>24; *x++ = y>> 16; *x++=y>>8; *x++ =y;
 static void
-DEFUN (build_bytes, (this_try, operand),
-       opcode_entry_type * this_try AND
-       struct z8k_op *operand)
+build_bytes (this_try, operand)
+     opcode_entry_type * this_try;
+     struct z8k_op *operand;
 {
   unsigned int i;
 
@@ -818,7 +1021,7 @@ DEFUN (build_bytes, (this_try, operand),
   char high;
   int nib;
   int nibble;
-  unsigned short *class_ptr;
+  unsigned int *class_ptr;
 
   frag_wane (frag_now);
   frag_new (0);
@@ -852,30 +1055,60 @@ top:;
 	  /* pc rel 8 bit */
 	  output_ptr = apply_fix (output_ptr, R_JR, da_operand, 2);
 	  da_operand = 0;
-
 	  break;
+
+	case CLASS_0DISP7:
+	  /* pc rel 7 bit */
+	  *output_ptr = 0;
+	  output_ptr = apply_fix (output_ptr, R_DISP7, da_operand, 2);
+	  da_operand = 0;
+	  break;
+
+	case CLASS_1DISP7:
+	  /* pc rel 7 bit */
+	  *output_ptr = 0x80;
+	  output_ptr = apply_fix (output_ptr, R_DISP7, da_operand, 2);
+          output_ptr[-2] =  0x8;
+	  da_operand = 0;
+	  break;
+
 	case CLASS_BIT_1OR2:
 	  *output_ptr = c & 0xf;
-	  if (imm_operand)  
-	  {
-	    if (imm_operand->X_add_number==2)
+	  if (imm_operand)
 	    {
-	      *output_ptr |= 2;
+	      if (imm_operand->X_add_number == 2)
+		{
+		  *output_ptr |= 2;
+		}
+	      else if (imm_operand->X_add_number != 1)
+		{
+		  as_bad ("immediate must be 1 or 2");
+		}
 	    }
-	    else if (imm_operand->X_add_number != 1)
+	  else
 	    {
-	      as_bad("immediate must be 1 or 2");
+	      as_bad ("immediate 1 or 2 expected");
 	    }
-	  }
-	  else 
-	  {
-	    as_bad("immediate 1 or 2 expected");
-	  }
 	  output_ptr++;
 	  break;
 	case CLASS_CC:
 	  *output_ptr++ = the_cc;
 	  break;
+        case CLASS_0CCC:
+          *output_ptr++ = the_ctrl;
+          break;
+        case CLASS_1CCC:
+          *output_ptr++ = the_ctrl | 0x8;
+          break;
+        case CLASS_00II:
+          *output_ptr++ = (~the_interrupt & 0x3);
+          break;
+        case CLASS_01II:
+          *output_ptr++ = (~the_interrupt & 0x3) | 0x4;
+          break;
+        case CLASS_FLAGS:
+          *output_ptr++ = the_flags;
+          break;
 	case CLASS_BIT:
 	  *output_ptr++ = c & 0xf;
 	  break;
@@ -897,7 +1130,7 @@ top:;
 	  output_ptr = apply_fix (output_ptr, R_IMM16, da_operand, 4);
 	  da_operand = 0;
 	  break;
-	  
+
 	case CLASS_IMM:
 	  {
 	    nib = 0;
@@ -973,21 +1206,21 @@ DEFUN (md_assemble, (str),
 
   /* Drop leading whitespace */
   while (*str == ' ')
-   str++;
+    str++;
 
   /* find the op code end */
   for (op_start = op_end = str;
        *op_end != 0 && *op_end != ' ';
        op_end++)
-  {
-  }
+    {
+    }
 
   ;
 
   if (op_end == op_start)
-  {
-    as_bad ("can't find opcode ");
-  }
+    {
+      as_bad ("can't find opcode ");
+    }
   c = *op_end;
 
   *op_end = 0;
@@ -997,56 +1230,56 @@ DEFUN (md_assemble, (str),
 
 
   if (opcode == NULL)
-  {
-    as_bad ("unknown opcode");
-    return;
-  }
-
-  if (opcode->opcode == 250) 
-  {
-    /* was really a pseudo op */
-
-    pseudo_typeS *p ;
-    char oc;
-
-    char *old = input_line_pointer;
-    *op_end = c;
-
-     
-    input_line_pointer = op_end;
-
-    oc = *old;
-    *old = '\n';
-    while (*input_line_pointer == ' ')
-     input_line_pointer++;
-    p = (pseudo_typeS *)(opcode->func);
-
-    (p->poc_handler)(p->poc_val);
-    input_line_pointer = old;
-    *old = oc;
-  }
-  else {
-    input_line_pointer = get_operands (opcode, op_end,
-				       operand);
-    *op_end = c;
-    prev_opcode = opcode;
-
-    opcode = get_specific (opcode, operand);
-
-    if (opcode == 0)
     {
-      /* Couldn't find an opcode which matched the operands */
-      char *where = frag_more (2);
-
-      where[0] = 0x0;
-      where[1] = 0x0;
-
-      as_bad ("Can't find opcode to match operands");
+      as_bad ("unknown opcode");
       return;
     }
 
-    build_bytes (opcode, operand);
-  }
+  if (opcode->opcode == 250)
+    {
+      /* was really a pseudo op */
+
+      pseudo_typeS *p;
+      char oc;
+
+      char *old = input_line_pointer;
+      *op_end = c;
+
+
+      input_line_pointer = op_end;
+
+      oc = *old;
+      *old = '\n';
+      while (*input_line_pointer == ' ')
+	input_line_pointer++;
+      p = (pseudo_typeS *) (opcode->func);
+
+      (p->poc_handler) (p->poc_val);
+      input_line_pointer = old;
+      *old = oc;
+    }
+  else
+    {
+      input_line_pointer = get_operands (opcode, op_end,
+					 operand);
+      prev_opcode = opcode;
+
+      opcode = get_specific (opcode, operand);
+
+      if (opcode == 0)
+	{
+	  /* Couldn't find an opcode which matched the operands */
+	  char *where = frag_more (2);
+
+	  where[0] = 0x0;
+	  where[1] = 0x0;
+
+	  as_bad ("Can't find opcode to match operands");
+	  return;
+	}
+
+      build_bytes (opcode, operand);
+    }
 }
 
 void
@@ -1070,11 +1303,6 @@ DEFUN (tc_headers_hook, (headers),
   printf ("call to tc_headers_hook \n");
 }
 
-void
-DEFUN_VOID (md_end)
-{
-}
-
 /* Various routines to kill one day */
 /* Equal to MAX_PRECISION in atof-ieee.c */
 #define MAX_LITTLENUMS 6
@@ -1135,7 +1363,7 @@ md_atof (type, litP, sizeP)
       md_number_to_chars (litP, (long) (*wordP++), sizeof (LITTLENUM_TYPE));
       litP += sizeof (LITTLENUM_TYPE);
     }
-  return "";			/* Someone should teach Dean about null pointers */
+  return 0;
 }
 
 int
@@ -1145,13 +1373,16 @@ md_parse_option (argP, cntP, vecP)
      char ***vecP;
 
 {
-  if (!strcmp(*argP,"z8001")) {
-    s_segm();
-  }
-  else if (!strcmp(*argP,"z8002")) {
-    s_unseg();
-  }
-  else return 0;
+  if (!strcmp (*argP, "z8001"))
+    {
+      s_segm ();
+    }
+  else if (!strcmp (*argP, "z8002"))
+    {
+      s_unseg ();
+    }
+  else
+    return 0;
   **argP = 0;
   return 1;
 }
@@ -1168,8 +1399,8 @@ tc_aout_fix_to_chars ()
 void
 md_create_short_jump (ptr, from_addr, to_addr, frag, to_symbol)
      char *ptr;
-     long from_addr;
-     long to_addr;
+     addressT from_addr;
+     addressT to_addr;
      fragS *frag;
      symbolS *to_symbol;
 {
@@ -1179,7 +1410,7 @@ md_create_short_jump (ptr, from_addr, to_addr, frag, to_symbol)
 void
 md_create_long_jump (ptr, from_addr, to_addr, frag, to_symbol)
      char *ptr;
-     long from_addr, to_addr;
+     addressT from_addr, to_addr;
      fragS *frag;
      symbolS *to_symbol;
 {
@@ -1196,10 +1427,10 @@ md_convert_frag (headers, fragP)
   abort ();
 }
 
-long
+valueT
 DEFUN (md_section_align, (seg, size),
        segT seg AND
-       long size)
+       valueT size)
 {
   return ((size + (1 << section_alignment[(int) seg]) - 1) & (-1 << section_alignment[(int) seg]));
 
@@ -1224,10 +1455,16 @@ md_apply_fix (fixP, val)
       /*    if (val != 0) abort();*/
       break;
 
+    case R_DISP7:
+
+      *buf++ += val;
+      /*    if (val != 0) abort();*/
+      break;
+
     case R_IMM8:
       buf[0] += val;
       break;
-     case R_IMM16:
+    case R_IMM16:
       *buf++ = (val >> 8);
       *buf++ = val;
       break;
@@ -1272,7 +1509,7 @@ md_estimate_size_before_relax (fragP, segment_type)
 void
 DEFUN (md_number_to_chars, (ptr, use, nbytes),
        char *ptr AND
-       long use AND
+       valueT use AND
        int nbytes)
 {
   switch (nbytes)
@@ -1311,43 +1548,57 @@ tc_reloc_mangle (fix_ptr, intr, base)
 {
   symbolS *symbol_ptr;
 
+  if (fix_ptr->fx_addsy &&
+      fix_ptr->fx_subsy) 
+    {
+      symbolS *add = fix_ptr->fx_addsy;
+      symbolS *sub = fix_ptr->fx_subsy;
+      if (S_GET_SEGMENT(add) != S_GET_SEGMENT(sub))
+	{
+	  as_bad("Can't subtract symbols in different sections %s %s",
+		 S_GET_NAME(add), S_GET_NAME(sub));
+	}
+      else {
+	int diff = S_GET_VALUE(add) - S_GET_VALUE(sub);
+	fix_ptr->fx_addsy = 0;
+	fix_ptr->fx_subsy = 0;
+	fix_ptr->fx_offset += diff;
+      }
+    }
   symbol_ptr = fix_ptr->fx_addsy;
 
   /* If this relocation is attached to a symbol then it's ok
      to output it */
   if (fix_ptr->fx_r_type == 0)
-  {
-    /* cons likes to create reloc32's whatever the size of the reloc..
-     */
-    switch (fix_ptr->fx_size)
     {
-
-     case 2:
-      intr->r_type = R_IMM16;
-      break;
-     case 1:
-      intr->r_type = R_IMM8;
-      break;
-     case 4:
-      intr->r_type = R_IMM32;
-      break;
-     default:
-      abort ();
+      /* cons likes to create reloc32's whatever the size of the reloc.. */
+      switch (fix_ptr->fx_size)
+	{
+	case 2:
+	  intr->r_type = R_IMM16;
+	  break;
+	case 1:
+	  intr->r_type = R_IMM8;
+	  break;
+	case 4:
+	  intr->r_type = R_IMM32;
+	  break;
+	default:
+	  abort ();
+	}
 
     }
-
-  }
   else
-  {
-    intr->r_type = fix_ptr->fx_r_type;
-  }
+    {
+      intr->r_type = fix_ptr->fx_r_type;
+    }
 
   intr->r_vaddr = fix_ptr->fx_frag->fr_address + fix_ptr->fx_where + base;
   intr->r_offset = fix_ptr->fx_offset;
 
   if (symbol_ptr)
-   intr->r_symndx = symbol_ptr->sy_number;
+    intr->r_symndx = symbol_ptr->sy_number;
   else
-   intr->r_symndx = -1;
-
+    intr->r_symndx = -1;
 }
+
