@@ -38,13 +38,12 @@ frag_init ()
 /*
  *			frag_grow()
  *
- * Internal.
  * Try to augment current frag by nchars chars.
  * If there is no room, close of the current frag with a ".fill 0"
  * and begin a new frag. Unless the new frag has nchars chars available
  * do not return. Do not set up any fields of *now_frag.
  */
-static void 
+void 
 frag_grow (nchars)
      unsigned int nchars;
 {
@@ -197,7 +196,7 @@ frag_var (type, max_chars, var, subtype, symbol, offset, opcode)
   frag_now->fr_bsr = 0;
   frag_new (max_chars);
   return (retval);
-}				/* frag_var() */
+}
 
 /*
  *			frag_variant()
@@ -264,8 +263,22 @@ frag_align (alignment, fill_character)
      int alignment;
      int fill_character;
 {
-  *(frag_var (rs_align, 1, 1, (relax_substateT) 0, (symbolS *) 0,
-	      (long) alignment, (char *) 0)) = fill_character;
-}				/* frag_align() */
+  char *p;
+  p = frag_var (rs_align, 1, 1, (relax_substateT) 0,
+		(symbolS *) 0, (long) alignment, (char *) 0);
+  *p = fill_character;
+}
+
+void 
+frag_align_pattern (alignment, fill_pattern, n_fill)
+     int alignment;
+     const char *fill_pattern;
+     int n_fill;
+{
+  char *p;
+  p = frag_var (rs_align, n_fill, n_fill, (relax_substateT) 0,
+		(symbolS *) 0, (long) alignment, (char *) 0);
+  memcpy (p, fill_pattern, n_fill);
+}
 
 /* end of frags.c */
