@@ -96,6 +96,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
 #include "as.h"
 #include <obstack.h>
 #include "input-file.h"
+#include "subsegs.h"
 
 #ifndef NO_LISTING
 #ifndef LISTING_HEADER
@@ -180,7 +181,6 @@ list_info_type;
 static struct list_info_struct *head;
 struct list_info_struct *listing_tail;
 extern int listing;
-extern unsigned int physical_input_line;
 extern fragS *frag_now;
 
 
@@ -199,8 +199,8 @@ static unsigned int data_buffer_size;
 
 
 /* Prototypes.  */
-static void listing_message PARAMS ((char *name, char *message));
-static file_info_type *file_info PARAMS ((char *file_name));
+static void listing_message PARAMS ((const char *name, const char *message));
+static file_info_type *file_info PARAMS ((const char *file_name));
 static void new_frag PARAMS ((void));
 static char *buffer_line PARAMS ((file_info_type *file,
 				  char *line, unsigned int size));
@@ -220,8 +220,8 @@ static void listing_listing PARAMS ((char *name));
 
 static void
 listing_message (name, message)
-     char *name;
-     char *message;
+     const char *name;
+     const char *message;
 {
   unsigned int l = strlen (name) + strlen (message) + 1;
   char *n = (char *) xmalloc (l);
@@ -231,22 +231,18 @@ listing_message (name, message)
     {
       listing_tail->message = n;
     }
-
 }
 
-
-
-
 void
 listing_warning (message)
-     char *message;
+     const char *message;
 {
   listing_message ("Warning:", message);
 }
 
 void
 listing_error (message)
-     char *message;
+     const char *message;
 {
   listing_message ("Error:", message);
 }
@@ -258,7 +254,7 @@ static file_info_type *file_info_head;
 
 static file_info_type *
 file_info (file_name)
-     char *file_name;
+     const char *file_name;
 {
   /* Find an entry with this file name */
   file_info_type *p = file_info_head;
@@ -275,19 +271,17 @@ file_info (file_name)
   p = (file_info_type *) xmalloc (sizeof (file_info_type));
   p->next = file_info_head;
   file_info_head = p;
-  p->filename = xmalloc (strlen (file_name) + 1);
+  p->filename = xmalloc ((unsigned long) strlen (file_name) + 1);
   strcpy (p->filename, file_name);
   p->linenum = 0;
   p->end_pending = 0;
 
-  p->file = fopen (p->filename, "rb");
+  /* Do we really prefer binary mode for this??  */
+  p->file = fopen (p->filename, FOPEN_RB);
   if (p->file)
     fgetc (p->file);
 
-
-
   return p;
-
 }
 
 
@@ -304,20 +298,21 @@ void
 listing_newline (ps)
      char *ps;
 {
-  extern char *file_name;
+  char *file;
+  unsigned int line;
   static unsigned int last_line = 0xffff;
-
-
   list_info_type *new;
-  if (physical_input_line != last_line)
+
+  as_where (&file, &line);
+  if (line != last_line)
     {
-      last_line = physical_input_line;
+      last_line = line;
       new_frag ();
 
       new = (list_info_type *) xmalloc (sizeof (list_info_type));
       new->frag = frag_now;
-      new->line = physical_input_line;
-      new->file = file_info (file_name);
+      new->line = line;
+      new->file = file_info (file);
 
       if (listing_tail)
 	{
@@ -337,6 +332,34 @@ listing_newline (ps)
     }
 }
 
+/* Attach all current frags to the previous line instead of the
+   current line.  This is called by the MIPS backend when it discovers
+   that it needs to add some NOP instructions; the added NOP
+   instructions should go with the instruction that has the delay, not
+   with the new instruction.  */
+
+void
+listing_prev_line ()
+{
+  list_info_type *l;
+  fragS *f;
+
+  if (head == (list_info_type *) NULL
+      || head == listing_tail)
+    return;
+
+  new_frag ();
+
+  for (l = head; l->next != listing_tail; l = l->next)
+    ;
+
+  for (f = frchain_now->frch_root; f != (fragS *) NULL; f = f->fr_next)
+    if (f->line == listing_tail)
+      f->line = l;
+
+  listing_tail->frag = frag_now;
+  new_frag ();
+}
 
 /*
  This function returns the next source line from the file supplied,
@@ -398,7 +421,7 @@ buffer_line (file, line, size)
 }
 
 
-static char *fn;
+static const char *fn;
 
 static unsigned int eject;	/* Eject pending */
 static unsigned int page;	/* Current page number */
@@ -457,12 +480,12 @@ calc_hex (list)
      list_info_type * list;
 {
   list_info_type *first = list;
-  unsigned int address = ~0;
+  unsigned int address = (unsigned int) ~0;
 
   fragS *frag;
   fragS *frag_ptr;
 
-  unsigned int byte_in_frag = 0;
+  unsigned int byte_in_frag;
 
 
   /* Find first frag which says it belongs to this line */
@@ -478,6 +501,7 @@ calc_hex (list)
   while (frag_ptr != (fragS *) NULL && frag_ptr->line == first)
     {
       /* Print as many bytes from the fixed part as is sensible */
+      byte_in_frag = 0;
       while (byte_in_frag < frag_ptr->fr_fix && data_buffer_size < sizeof (data_buffer) - 10)
 	{
 	  if (address == ~0)
@@ -634,20 +658,15 @@ print_lines (list, string, address)
 }
 
 
-
-
-
-
 static void
 list_symbol_table ()
 {
   extern symbolS *symbol_rootP;
+  int got_some = 0;
 
   symbolS *ptr;
   eject = 1;
   listing_page (0);
-  printf ("DEFINED SYMBOLS\n");
-  on_page++;
 
   for (ptr = symbol_rootP; ptr != (symbolS *) NULL; ptr = symbol_next (ptr))
     {
@@ -655,53 +674,81 @@ list_symbol_table ()
 	{
 	  if (S_GET_NAME (ptr))
 	    {
+	      char buf[30];
+	      valueT val = S_GET_VALUE (ptr);
 
-	      if (strlen (S_GET_NAME (ptr)))
+	      /* @@ Note that this is dependent on the compilation options,
+		 not solely on the target characteristics.  */
+	      if (sizeof (val) == 4 && sizeof (int) == 4)
+		sprintf (buf, "%08lx", (unsigned long) val);
+#if defined (BFD64)
+	      else if (sizeof (val) > 4)
 		{
-		  printf ("%20s:%-5d  %2d:%08x %s \n",
-			  ptr->sy_frag->line->file->filename,
-			  ptr->sy_frag->line->line,
-			  S_GET_SEGMENT (ptr),
-			  S_GET_VALUE (ptr),
-			  S_GET_NAME (ptr));
+		  char buf1[30];
+		  sprintf_vma (buf1, val);
+		  strcpy (buf, "00000000");
+		  strcpy (buf + 8 - strlen (buf1), buf1);
 		}
-
+#endif
 	      else
+		abort ();
+
+	      if (!got_some)
 		{
-		  printf ("%20s:%-5d  %2d:%08x\n",
-			  ptr->sy_frag->line->file->filename,
-			  ptr->sy_frag->line->line,
-			  S_GET_SEGMENT (ptr),
-			  S_GET_VALUE (ptr));
-
-
+		  printf ("DEFINED SYMBOLS\n");
+		  on_page++;
+		  got_some = 1;
 		}
 
+	      printf ("%20s:%-5d  %s:%s %s\n",
+		      ptr->sy_frag->line->file->filename,
+		      ptr->sy_frag->line->line,
+		      segment_name (S_GET_SEGMENT (ptr)),
+		      buf, S_GET_NAME (ptr));
+
 	      on_page++;
 	      listing_page (0);
 	    }
 	}
 
     }
+  if (!got_some)
+    {
+      printf ("NO DEFINED SYMBOLS\n");
+      on_page++;
+    }
   printf ("\n");
   on_page++;
   listing_page (0);
-  printf ("UNDEFINED SYMBOLS\n");
-  on_page++;
-  listing_page (0);
+
+  got_some = 0;
 
   for (ptr = symbol_rootP; ptr != (symbolS *) NULL; ptr = symbol_next (ptr))
     {
       if (S_GET_NAME (ptr) && strlen (S_GET_NAME (ptr)) != 0)
 	{
-	  if (ptr->sy_frag->line == 0)
+	  if (ptr->sy_frag->line == 0
+	      && S_GET_SEGMENT (ptr) != reg_section)
 	    {
+	      if (!got_some)
+		{
+		  got_some = 1;
+		  printf ("UNDEFINED SYMBOLS\n");
+		  on_page++;
+		  listing_page (0);
+		}
 	      printf ("%s\n", S_GET_NAME (ptr));
 	      on_page++;
 	      listing_page (0);
 	    }
 	}
     }
+  if (!got_some)
+    {
+      printf ("NO UNDEFINED SYMBOLS\n");
+      on_page++;
+      listing_page (0);
+    }
 }
 
 static void
@@ -887,19 +934,21 @@ listing_print (name)
 
 void
 listing_file (name)
-     char *name;
+     const char *name;
 {
   fn = name;
 }
 
 void
-listing_eject ()
+listing_eject (ignore)
+     int ignore;
 {
   listing_tail->edict = EDICT_EJECT;
 }
 
 void
-listing_flags ()
+listing_flags (ignore)
+     int ignore;
 {
   while ((*input_line_pointer++) && (*input_line_pointer != '\n'))
     input_line_pointer++;
@@ -908,14 +957,15 @@ listing_flags ()
 
 void
 listing_list (on)
-     unsigned int on;
+     int on;
 {
   listing_tail->edict = on ? EDICT_LIST : EDICT_NOLIST;
 }
 
 
 void
-listing_psize ()
+listing_psize (ignore)
+     int ignore;
 {
   paper_height = get_absolute_expression ();
 
@@ -934,10 +984,10 @@ listing_psize ()
 
 void
 listing_title (depth)
-     unsigned int depth;
+     int depth;
 {
   char *start;
-  char *title;
+  char *ttl;
   unsigned int length;
 
   SKIP_WHITESPACE ();
@@ -951,11 +1001,11 @@ listing_title (depth)
 	  if (*input_line_pointer == '\"')
 	    {
 	      length = input_line_pointer - start;
-	      title = xmalloc (length + 1);
-	      memcpy (title, start, length);
-	      title[length] = 0;
+	      ttl = xmalloc (length + 1);
+	      memcpy (ttl, start, length);
+	      ttl[length] = 0;
 	      listing_tail->edict = depth ? EDICT_SBTTL : EDICT_TITLE;
-	      listing_tail->edict_arg = title;
+	      listing_tail->edict_arg = ttl;
 	      input_line_pointer++;
 	      demand_empty_rest_of_line ();
 	      return;
@@ -992,9 +1042,10 @@ listing_source_line (line)
 
 void
 listing_source_file (file)
-     char *file;
+     const char *file;
 {
-  listing_tail->hll_file = file_info (file);
+  if (listing_tail)
+    listing_tail->hll_file = file_info (file);
 }
 
 
@@ -1005,40 +1056,43 @@ listing_source_file (file)
 /* Dummy functions for when compiled without listing enabled */
 
 void
-listing_flags ()
+listing_flags (ignore)
+     int ignore;
 {
   s_ignore (0);
 }
 
 void 
 listing_list (on)
-     unsigned int on;
+     int on;
 {
   s_ignore (0);
 }
 
 void 
-listing_eject ()
+listing_eject (ignore)
+     int ignore;
 {
   s_ignore (0);
 }
 
 void 
-listing_psize ()
+listing_psize (ignore)
+     int ignore;
 {
   s_ignore (0);
 }
 
 void 
 listing_title (depth)
-     unsigned int depth;
+     int depth;
 {
   s_ignore (0);
 }
 
 void
 listing_file (name)
-     char *name;
+     const char *name;
 {
 
 }
@@ -1058,7 +1112,7 @@ listing_source_line (n)
 }
 void 
 listing_source_file (n)
-     char *n;
+     const char *n;
 {
 
 }
