@@ -1,5 +1,6 @@
 /* read.c - read a source file -
-   Copyright (C) 1986, 1987, 1990, 1991, 1993 Free Software Foundation, Inc.
+   Copyright (C) 1986, 1987, 1990, 1991, 1993, 1994
+   Free Software Foundation, Inc.
 
 This file is part of GAS, the GNU Assembler.
 
@@ -45,20 +46,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
 #include "obstack.h"
 #include "listing.h"
 
-/* We need this, despite the apparent object format dependency, since
-   it defines stab types, which all object formats can use now. */
-
-#include "aout/stab_gnu.h"
-
-/* Allow backends to override the names used for the stab sections.  */
-#ifndef STAB_SECTION_NAME
-#define STAB_SECTION_NAME ".stab"
-#endif
-
-#ifndef STAB_STRING_SECTION_NAME
-#define STAB_STRING_SECTION_NAME ".stabstr"
-#endif
-
 #ifndef TC_START_LABEL
 #define TC_START_LABEL(x,y) (x==':')
 #endif
@@ -84,17 +71,27 @@ die horribly;
 #define LEX_AT 0
 #endif
 
+#ifndef LEX_BR
+/* The RS/6000 assembler uses {,},[,] as parts of symbol names.  */
+#define LEX_BR 0
+#endif
+
+#ifndef LEX_PCT
+/* The Delta 68k assembler permits % inside label names.  */
+#define LEX_PCT 0
+#endif
+
 /* used by is_... macros. our ctype[] */
 const char lex_type[256] =
 {
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* @ABCDEFGHIJKLMNO */
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,	/* PQRSTUVWXYZ[\]^_ */
-  0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0,	/* _!"#$%&'()*+,-./ */
+  0, 0, 0, 0, 3, LEX_PCT, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0,	/* _!"#$%&'()*+,-./ */
   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,	/* 0123456789:;<=>? */
   LEX_AT, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,	/* @ABCDEFGHIJKLMNO */
-  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 0, 0, 0, 3,	/* PQRSTUVWXYZ[\]^_ */
+  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, LEX_BR, 0, LEX_BR, 0, 3, /* PQRSTUVWXYZ[\]^_ */
   0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,	/* `abcdefghijklmno */
-  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 0, 0, 0, 0,	/* pqrstuvwxyz{|}~. */
+  3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, LEX_BR, 0, LEX_BR, 0, 0, /* pqrstuvwxyz{|}~. */
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -160,7 +157,7 @@ struct broken_word *broken_words;
 int new_broken_words;
 #endif
 
-static char *demand_copy_string PARAMS ((int *lenP));
+char *demand_copy_string PARAMS ((int *lenP));
 int is_it_end_of_statement PARAMS ((void));
 static segT get_segmented_expression PARAMS ((expressionS *expP));
 static segT get_known_segmented_expression PARAMS ((expressionS * expP));
@@ -256,12 +253,20 @@ static const pseudo_typeS potable[] =
   {"string", stringer, 1},
 /* tag */
   {"text", s_text, 0},
+
+  /* This is for gcc to use.  It's only just been added (2/94), so gcc
+     won't be able to use it for a while -- probably a year or more.
+     But once this has been released, check with gcc maintainers
+     before deleting it or even changing the spelling.  */
+  {"this_GCC_requires_the_GNU_assembler", s_ignore, 0},
+
   {"title", listing_title, 0},	/* Listing title */
 /* type */
 /* use */
 /* val */
   {"xstabs", s_xstab, 's'},
   {"word", cons, 2},
+  {"zero", s_space, 0},
   {NULL}			/* end sentinel */
 };
 
@@ -370,12 +375,14 @@ read_a_source_file (name)
 	   * If input_line_pointer [-1] == '\n' then we just
 	   * scanned another line: so bump line counters.
 	   */
-	  if (input_line_pointer[-1] == '\n')
+	  if (is_end_of_line[(unsigned char) input_line_pointer[-1]])
 	    {
-	      bump_line_counters ();
+	      if (input_line_pointer[-1] == '\n')
+		bump_line_counters ();
 
-#ifdef MRI
-	      /* Text at the start of a line must be a label, we run down and stick a colon in */
+#if defined (MRI) || defined (LABELS_WITHOUT_COLONS)
+	      /* Text at the start of a line must be a label, we run down
+		 and stick a colon in.  */
 	      if (is_name_beginner (*input_line_pointer))
 		{
 		  char *line_start = input_line_pointer;
@@ -418,7 +425,8 @@ read_a_source_file (name)
 	   * Input_line_pointer points after that character.
 	   */
 	  if (is_name_beginner (c))
-	    {			/* want user-defined label or pseudo/opcode */
+	    {
+	      /* want user-defined label or pseudo/opcode */
 	      HANDLE_CONDITIONAL_ASSEMBLY ();
 
 	      s = --input_line_pointer;
@@ -497,7 +505,7 @@ read_a_source_file (name)
 			  as_bad ("Unknown pseudo-op:  `%s'", s);
 			  *input_line_pointer = c;
 			  s_ignore (0);
-			  break;
+			  continue;
 			}
 
 		      /* Put it back for error messages etc. */
@@ -512,13 +520,7 @@ read_a_source_file (name)
 		       * Input_line_pointer->1st non-blank char
 		       * after pseudo-operation.
 		       */
-		      if (!pop)
-			{
-			  ignore_rest_of_line ();
-			  break;
-			}
-		      else
-			(*pop->poc_handler) (pop->poc_val);
+		      (*pop->poc_handler) (pop->poc_val);
 		    }
 		  else
 #endif
@@ -644,7 +646,7 @@ read_a_source_file (name)
 			num = buffer_limit - buffer;
 
 		      tmp_buf = xrealloc (tmp_buf, tmp_len + num);
-		      memcpy (tmp_buf, buffer + tmp_len, num);
+		      memcpy (tmp_buf + tmp_len, buffer, num);
 		      tmp_len += num;
 		    }
 		  while (!ends);
@@ -713,14 +715,49 @@ read_a_source_file (name)
     }				/* while (more buffers to scan) */
   input_scrub_close ();		/* Close the input file */
 
-}				/* read_a_source_file() */
+}
 
 void 
 s_abort (ignore)
      int ignore;
 {
   as_fatal (".abort detected.  Abandoning ship.");
-}				/* s_abort() */
+}
+
+/* Guts of .align directive.  */
+static void 
+do_align (n, fill)
+     int n;
+     char *fill;
+{
+#ifdef md_do_align
+  md_do_align (n, fill, just_record_alignment);
+#endif
+  if (!fill)
+    {
+      /* @@ Fix this right for BFD!  */
+      static char zero;
+      static char nop_opcode = NOP_OPCODE;
+
+      if (now_seg != data_section && now_seg != bss_section)
+	{
+	  fill = &nop_opcode;
+	}
+      else
+	{
+	  fill = &zero;
+	}
+    }
+  /* Only make a frag if we HAVE to. . . */
+  if (n && !need_pass_2)
+    frag_align (n, *fill);
+
+#ifdef md_do_align
+ just_record_alignment:
+#endif
+
+  record_alignment (now_seg, n);
+}
 
 /* For machines where ".align 4" means align to a 4 byte boundary. */
 void 
@@ -728,7 +765,7 @@ s_align_bytes (arg)
      int arg;
 {
   register unsigned int temp;
-  register long temp_fill;
+  char temp_fill;
   unsigned int i = 0;
   unsigned long max_alignment = 1 << 15;
 
@@ -742,10 +779,8 @@ s_align_bytes (arg)
       as_bad ("Alignment too large: %d. assumed.", temp = max_alignment);
     }
 
-  /*
-     * For the sparc, `.align (1<<n)' actually means `.align n'
-     * so we have to convert it.
-     */
+  /* For the sparc, `.align (1<<n)' actually means `.align n' so we
+     have to convert it.  */
   if (temp != 0)
     {
       for (i = 0; (temp & 1) == 0; temp >>= 1, ++i)
@@ -759,19 +794,13 @@ s_align_bytes (arg)
     {
       input_line_pointer++;
       temp_fill = get_absolute_expression ();
+      do_align (temp, &temp_fill);
     }
-  else if (now_seg != data_section && now_seg != bss_section)
-    temp_fill = NOP_OPCODE;
   else
-    temp_fill = 0;
-  /* Only make a frag if we HAVE to. . . */
-  if (temp && !need_pass_2)
-    frag_align ((int) temp, (int) temp_fill);
-
-  record_alignment (now_seg, (int) temp);
+    do_align (temp, (char *) 0);
 
   demand_empty_rest_of_line ();
-}				/* s_align_bytes() */
+}
 
 /* For machines where ".align 4" means align to 2**4 boundary. */
 void 
@@ -779,7 +808,7 @@ s_align_ptwo (ignore)
      int ignore;
 {
   register int temp;
-  register long temp_fill;
+  char temp_fill;
   long max_alignment = 15;
 
   temp = get_absolute_expression ();
@@ -794,20 +823,13 @@ s_align_ptwo (ignore)
     {
       input_line_pointer++;
       temp_fill = get_absolute_expression ();
+      do_align (temp, &temp_fill);
     }
-  /* @@ Fix this right for BFD!  */
-  else if (now_seg != data_section && now_seg != bss_section)
-    temp_fill = NOP_OPCODE;
   else
-    temp_fill = 0;
-  /* Only make a frag if we HAVE to. . . */
-  if (temp && !need_pass_2)
-    frag_align (temp, (int) temp_fill);
-
-  record_alignment (now_seg, temp);
+    do_align (temp, (char *) 0);
 
   demand_empty_rest_of_line ();
-}				/* s_align_ptwo() */
+}
 
 void 
 s_comm (ignore)
@@ -843,7 +865,8 @@ s_comm (ignore)
   *p = c;
   if (S_IS_DEFINED (symbolP))
     {
-      as_bad ("Ignoring attempt to re-define symbol");
+      as_bad ("Ignoring attempt to re-define symbol `%s'.",
+	      S_GET_NAME (symbolP));
       ignore_rest_of_line ();
       return;
     }
@@ -1059,14 +1082,14 @@ s_lcomm (needs_align)
   p = input_line_pointer;
   *p = c;
   SKIP_WHITESPACE ();
-  if (*input_line_pointer != ',')
-    {
-      as_bad ("Expected comma after name");
-      ignore_rest_of_line ();
-      return;
-    }
 
-  ++input_line_pointer;
+  /* Accept an optional comma after the name.  The comma used to be
+     required, but Irix 5 cc does not generate it.  */
+  if (*input_line_pointer == ',')
+    {
+      ++input_line_pointer;
+      SKIP_WHITESPACE ();
+    }
 
   if (*input_line_pointer == '\n')
     {
@@ -1081,13 +1104,30 @@ s_lcomm (needs_align)
       return;
     }
 
-#ifdef TC_MIPS
+#if defined (TC_MIPS) || defined (TC_ALPHA)
 #if defined (OBJ_ECOFF) || defined (OBJ_ELF)
-  /* For MIPS ECOFF or ELF, small objects are put in .sbss.  */
+  /* For MIPS and Alpha ECOFF or ELF, small objects are put in .sbss.  */
   if (temp <= bfd_get_gp_size (stdoutput))
-    bss_seg = subseg_new (".sbss", 1);
+    {
+      bss_seg = subseg_new (".sbss", 1);
+      seg_info (bss_seg)->bss = 1;
+    }
 #endif
 #endif
+   if (!needs_align)
+     {
+       /* FIXME. This needs to be machine independent. */
+       if (temp >= 8)
+	 align = 3;
+       else if (temp >= 4)
+	 align = 2;
+       else if (temp >= 2)
+	 align = 1;
+       else
+	 align = temp;
+
+       record_alignment(bss_seg, align);
+     }
 
   if (needs_align)
     {
@@ -1119,6 +1159,18 @@ s_lcomm (needs_align)
 	}
       record_alignment (bss_seg, align);
     }				/* if needs align */
+  else
+    {
+      /* Assume some objects may require alignment on some systems.  */
+#ifdef TC_ALPHA
+      if (temp > 1)
+	{
+	  align = ffs (temp) - 1;
+	  if (temp % (1 << align))
+	    abort ();
+	}
+#endif
+    }
 
   *p = 0;
   symbolP = symbol_find_or_make (name);
@@ -1160,9 +1212,8 @@ s_lcomm (needs_align)
 #endif /* OBJ_COFF */
     }
   else
-    {
-      as_bad ("Ignoring attempt to re-define symbol %s.", name);
-    }
+    as_bad ("Ignoring attempt to re-define symbol `%s'.",
+	    S_GET_NAME (symbolP));
 
   subseg_set (current_seg, current_subseg);
 
@@ -1600,7 +1651,7 @@ cons (nbytes)
 
   input_line_pointer--;		/* Put terminator back into stream. */
   demand_empty_rest_of_line ();
-}				/* cons() */
+}
 
 /* Put the contents of expression EXP into the object file using
    NBYTES bytes.  If need_pass_2 is 1, this does nothing.  */
@@ -1717,17 +1768,17 @@ emit_expr (exp, nbytes)
 
   if (op == O_constant)
     {
-      register long get;
-      register long use;
-      register long mask;
-      register long unmask;
+      register valueT get;
+      register valueT use;
+      register valueT mask;
+      register valueT unmask;
 
       /* JF << of >= number of bits in the object is undefined.  In
 	 particular SPARC (Sun 4) has problems */
-      if (nbytes >= sizeof (long))
+      if (nbytes >= sizeof (valueT))
 	mask = 0;
       else
-	mask = ~0 << (BITS_PER_CHAR * nbytes);	/* Don't store these bits. */
+	mask = ~(valueT) 0 << (BITS_PER_CHAR * nbytes);	/* Don't store these bits. */
 
       unmask = ~mask;		/* Do store these bits. */
 
@@ -1743,7 +1794,7 @@ emit_expr (exp, nbytes)
 	  as_warn ("Value 0x%lx truncated to 0x%lx.", get, use);
 	}
       /* put bytes in right order. */
-      md_number_to_chars (p, (valueT) use, (int) nbytes);
+      md_number_to_chars (p, use, (int) nbytes);
     }
   else if (op == O_big)
     {
@@ -2246,6 +2297,7 @@ next_char_of_string ()
       c = NOT_A_CHAR;
       break;
 
+#ifndef NO_STRING_ESCAPES
     case '\\':
       switch (c = *input_line_pointer++)
 	{
@@ -2340,6 +2392,7 @@ next_char_of_string ()
 	  break;
 	}			/* switch on escaped char */
       break;
+#endif /* ! defined (NO_STRING_ESCAPES) */
 
     default:
       break;
@@ -2452,7 +2505,7 @@ demand_copy_C_string (len_pointer)
  * Demand string, but return a safe (=private) copy of the string.
  * Return NULL if we can't read a string here.
  */
-static char *
+char *
 demand_copy_string (lenP)
      int *lenP;
 {
@@ -2562,7 +2615,7 @@ s_include (arg)
       strcpy (path, include_dirs[i]);
       strcat (path, "/");
       strcat (path, filename);
-      if (0 != (try = fopen (path, "r")))
+      if (0 != (try = fopen (path, FOPEN_RT)))
 	{
 	  fclose (try);
 	  goto gotit;
@@ -2611,355 +2664,7 @@ s_ignore (arg)
       ++input_line_pointer;
     }
   ++input_line_pointer;
-
-  return;
-}				/* s_ignore() */
-
-/*
- * Handle .stabX directives, which used to be open-coded.
- * So much creeping featurism overloaded the semantics that we decided
- * to put all .stabX thinking in one place. Here.
- *
- * We try to make any .stabX directive legal. Other people's AS will often
- * do assembly-time consistency checks: eg assigning meaning to n_type bits
- * and "protecting" you from setting them to certain values. (They also zero
- * certain bits before emitting symbols. Tut tut.)
- *
- * If an expression is not absolute we either gripe or use the relocation
- * information. Other people's assemblers silently forget information they
- * don't need and invent information they need that you didn't supply.
- */
-
-/*
- * Build a string dictionary entry for a .stabX symbol.
- * The symbol is added to the .<secname>str section.
- */
-
-#ifdef SEPARATE_STAB_SECTIONS
-
-unsigned int
-get_stab_string_offset (string, stabstr_secname)
-     const char *string;
-     const char *stabstr_secname;
-{
-  unsigned int length;
-  unsigned int retval;
-
-  retval = 0;
-  length = strlen (string);
-  if (length > 0)
-    {				/* Ordinary case. */
-      segT save_seg;
-      subsegT save_subseg;
-      char *newsecname;
-      segT seg;
-      char *p;
-
-      save_seg = now_seg;
-      save_subseg = now_subseg;
-
-      /* Create the stab string section.  */
-      newsecname = xmalloc ((unsigned long) (strlen (stabstr_secname) + 1));
-      strcpy (newsecname, stabstr_secname);
-
-      seg = subseg_new (newsecname, 0);
-
-      retval = seg_info (seg)->stabu.stab_string_size;
-      if (retval > 0)
-	free (newsecname);
-      else
-	{
-	  /* Make sure the first string is empty.  */
-	  p = frag_more (1);
-	  *p = 0;
-	  retval = seg_info (seg)->stabu.stab_string_size = 1;
-#ifdef BFD_ASSEMBLER
-	  bfd_set_section_flags (stdoutput, seg, SEC_READONLY | SEC_DEBUGGING);
-#else
-	  free (newsecname);
-#endif
-	}
-
-      p = frag_more (length + 1);
-      strcpy (p, string);
-
-      seg_info (seg)->stabu.stab_string_size += length + 1;
-
-      subseg_set (save_seg, save_subseg);
-    }
-
-  return retval;
 }
 
-#endif /* SEPARATE_STAB_SECTIONS */
-
-/* This can handle different kinds of stabs (s,n,d) and different
-   kinds of stab sections. */
-
-static void 
-s_stab_generic (what, stab_secname, stabstr_secname)
-     int what;
-     char *stab_secname;
-     char *stabstr_secname;
-{
-  long longint;
-  char *string;
-  int type;
-  int other;
-  int desc;
-
-  /* The general format is:
-     .stabs "STRING",TYPE,OTHER,DESC,VALUE
-     .stabn TYPE,OTHER,DESC,VALUE
-     .stabd TYPE,OTHER,DESC
-     At this point input_line_pointer points after the pseudo-op and
-     any trailing whitespace.  The argument what is one of 's', 'n' or
-     'd' indicating which type of .stab this is.  */
-
-  if (what != 's')
-    string = "";
-  else
-    {
-      int length;
-
-      string = demand_copy_C_string (&length);
-      SKIP_WHITESPACE ();
-      if (*input_line_pointer == ',')
-	input_line_pointer++;
-      else
-	{
-	  as_warn (".stabs: Missing comma");
-	  ignore_rest_of_line ();
-	  return;
-	}
-    }
-
-  if (get_absolute_expression_and_terminator (&longint) != ',')
-    {
-      as_warn (".stab%c: Missing comma", what);
-      ignore_rest_of_line ();
-      return;
-    }
-  type = longint;
-
-  if (get_absolute_expression_and_terminator (&longint) != ',')
-    {
-      as_warn (".stab%c: Missing comma", what);
-      ignore_rest_of_line ();
-      return;
-    }
-  other = longint;
-
-  desc = get_absolute_expression ();
-  if (what == 's' || what == 'n')
-    {
-      if (*input_line_pointer != ',')
-	{
-	  as_warn (".stab%c: Missing comma", what);
-	  ignore_rest_of_line ();
-	  return;
-	}
-      input_line_pointer++;
-      SKIP_WHITESPACE ();
-    }
-
-  /* We have now gathered the type, other, and desc information.  For
-     .stabs or .stabn, input_line_pointer is now pointing at the
-     value.  */
-
-#ifdef SEPARATE_STAB_SECTIONS
-  /* Output the stab information in a separate section.  This is used
-     at least for COFF and ELF.  */
-  {
-    segT saved_seg = now_seg;
-    subsegT saved_subseg = now_subseg;
-    fragS *saved_frag = frag_now;
-    valueT dot;
-    segT seg;
-    unsigned int stroff;
-    char *p;
-    
-    dot = frag_now_fix ();
-
-    seg = subseg_new (stab_secname, 0);
-
-    if (! seg_info (seg)->hadone)
-      {
-#ifdef BFD_ASSEMBLER
-	bfd_set_section_flags (stdoutput, seg,
-			       SEC_READONLY | SEC_RELOC | SEC_DEBUGGING);
-#endif
-#ifdef INIT_STAB_SECTION
-	INIT_STAB_SECTION (seg);
-#endif
-	seg_info (seg)->hadone = 1;
-      }
-
-    stroff = get_stab_string_offset (string, stabstr_secname);
-
-    /* At least for now, stabs in a special stab section are always
-       output as 12 byte blocks of information.  */
-    p = frag_more (8);
-    md_number_to_chars (p, (valueT) stroff, 4);
-    md_number_to_chars (p + 4, (valueT) type, 1);
-    md_number_to_chars (p + 5, (valueT) other, 1);
-    md_number_to_chars (p + 6, (valueT) desc, 2);
-
-    if (what == 's' || what == 'n')
-      {
-	/* Pick up the value from the input line.  */
-	cons (4);
-	input_line_pointer--;
-      }
-    else
-      {
-	const char *fake;
-	symbolS *symbol;
-	expressionS exp;
-
-	/* Arrange for a value representing the current location.  */
-	fake = FAKE_LABEL_NAME;
-	symbol = symbol_new (fake, saved_seg, dot, saved_frag);
-
-	exp.X_op = O_symbol;
-	exp.X_add_symbol = symbol;
-	exp.X_add_number = 0;
-
-	emit_expr (&exp, 4);
-      }
-
-#ifdef OBJ_PROCESS_STAB
-    OBJ_PROCESS_STAB (seg, string, stroff, type, other, desc);
-#endif
-
-    subseg_set (saved_seg, saved_subseg);
-  }
-#else /* ! SEPARATE_STAB_SECTIONS */
-#ifdef OBJ_PROCESS_STAB
-  OBJ_PROCESS_STAB (what, string, type, other, desc);
-#else
-  /* Put the stab information in the symbol table.  */
-  {
-    symbolS *symbol;
-
-    symbol = symbol_new (string, undefined_section, 0,
-			 (struct frag *) NULL);
-    if (what == 's' || what == 'n')
-      {
-	/* Pick up the value from the input line.  */
-	symbol->sy_frag = &zero_address_frag;
-	pseudo_set (symbol);
-      }
-    else
-      {
-	/* .stabd sets the name to NULL.  Why?  */
-	S_SET_NAME (symbol, NULL);
-	symbol->sy_frag = frag_now;
-	S_SET_VALUE (symbol, (valueT) frag_now_fix ());
-      }
-
-    S_SET_TYPE (symbol, type);
-    S_SET_OTHER (symbol, other);
-    S_SET_DESC (symbol, desc);
-  }
-#endif /* ! OBJ_PROCESS_STAB */
-#endif /* ! SEPARATE_STAB_SECTIONS */
-
-#ifndef NO_LISTING
-  if (listing)
-    {
-      switch (type)
-	{
-	case N_SLINE:
-	  listing_source_line ((unsigned int) desc);
-	  break;
-	case N_SO:
-	case N_SOL:
-	  listing_source_file (string);
-	  break;
-	}
-    }
-#endif /* ! NO_LISTING */
-
-  demand_empty_rest_of_line ();
-}
-
-/* Regular stab directive. */
-
-void
-s_stab (what)
-     int what;
-{
-  s_stab_generic (what, STAB_SECTION_NAME, STAB_STRING_SECTION_NAME);
-}
-
-/* "Extended stabs", used in Solaris only now. */
-
-void
-s_xstab (what)
-     int what;
-{
-  int length;
-  char *stab_secname, *stabstr_secname;
-
-  stab_secname = demand_copy_C_string (&length);
-  SKIP_WHITESPACE ();
-  if (*input_line_pointer == ',')
-    input_line_pointer++;
-  else
-    {
-      as_bad ("comma missing in .xstabs");
-      ignore_rest_of_line ();
-      return;
-    }
-
-  /* To get the name of the stab string section, simply .str to
-     the stab section name.  */
-  stabstr_secname = (char *) xmalloc (strlen (stab_secname) + 4);
-  strcpy (stabstr_secname, stab_secname);
-  strcat (stabstr_secname, "str");
-  s_stab_generic (what, stab_secname, stabstr_secname);
-  free (stabstr_secname);
-}
-
-#ifdef S_SET_DESC
-
-/* Frob invented at RMS' request. Set the n_desc of a symbol.  */
-
-void 
-s_desc (ignore)
-     int ignore;
-{
-  char *name;
-  char c;
-  char *p;
-  symbolS *symbolP;
-  int temp;
-
-  name = input_line_pointer;
-  c = get_symbol_end ();
-  p = input_line_pointer;
-  *p = c;
-  SKIP_WHITESPACE ();
-  if (*input_line_pointer != ',')
-    {
-      *p = 0;
-      as_bad ("Expected comma after name \"%s\"", name);
-      *p = c;
-      ignore_rest_of_line ();
-    }
-  else
-    {
-      input_line_pointer++;
-      temp = get_absolute_expression ();
-      *p = 0;
-      symbolP = symbol_find_or_make (name);
-      *p = c;
-      S_SET_DESC (symbolP, temp);
-    }
-  demand_empty_rest_of_line ();
-}				/* s_desc() */
-
-#endif /* defined (S_SET_DESC) */
 
 /* end of read.c */
